from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricConductivityUnits(Enum):
        """
            ElectricConductivityUnits enumeration
        """
        
        SiemensPerMeter = 'SiemensPerMeter'
        """
            
        """
        
        SiemensPerInch = 'SiemensPerInch'
        """
            
        """
        
        SiemensPerFoot = 'SiemensPerFoot'
        """
            
        """
        
        SiemensPerCentimeter = 'SiemensPerCentimeter'
        """
            
        """
        
        MicrosiemensPerCentimeter = 'MicrosiemensPerCentimeter'
        """
            
        """
        
        MillisiemensPerCentimeter = 'MillisiemensPerCentimeter'
        """
            
        """
        

class ElectricConductivityDto:
    """
    A DTO representation of a ElectricConductivity

    Attributes:
        value (float): The value of the ElectricConductivity.
        unit (ElectricConductivityUnits): The specific unit that the ElectricConductivity value is representing.
    """

    def __init__(self, value: float, unit: ElectricConductivityUnits):
        """
        Create a new DTO representation of a ElectricConductivity

        Parameters:
            value (float): The value of the ElectricConductivity.
            unit (ElectricConductivityUnits): The specific unit that the ElectricConductivity value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricConductivity
        """
        self.unit: ElectricConductivityUnits = unit
        """
        The specific unit that the ElectricConductivity value is representing
        """

    def to_json(self):
        """
        Get a ElectricConductivity DTO JSON object representing the current unit.

        :return: JSON object represents ElectricConductivity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SiemensPerMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricConductivity DTO from a json representation.

        :param data: The ElectricConductivity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SiemensPerMeter"}
        :return: A new instance of ElectricConductivityDto.
        :rtype: ElectricConductivityDto
        """
        return ElectricConductivityDto(value=data["value"], unit=ElectricConductivityUnits(data["unit"]))


class ElectricConductivity(AbstractMeasure):
    """
    Electrical conductivity or specific conductance is the reciprocal of electrical resistivity, and measures a material's ability to conduct an electric current.

    Args:
        value (float): The value.
        from_unit (ElectricConductivityUnits): The ElectricConductivity unit to create from, The default unit is SiemensPerMeter
    """
    def __init__(self, value: float, from_unit: ElectricConductivityUnits = ElectricConductivityUnits.SiemensPerMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__siemens_per_meter = None
        
        self.__siemens_per_inch = None
        
        self.__siemens_per_foot = None
        
        self.__siemens_per_centimeter = None
        
        self.__microsiemens_per_centimeter = None
        
        self.__millisiemens_per_centimeter = None
        

    def convert(self, unit: ElectricConductivityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricConductivityUnits = ElectricConductivityUnits.SiemensPerMeter) -> ElectricConductivityDto:
        """
        Get a new instance of ElectricConductivity DTO representing the current unit.

        :param hold_in_unit: The specific ElectricConductivity unit to store the ElectricConductivity value in the DTO representation.
        :type hold_in_unit: ElectricConductivityUnits
        :return: A new instance of ElectricConductivityDto.
        :rtype: ElectricConductivityDto
        """
        return ElectricConductivityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricConductivityUnits = ElectricConductivityUnits.SiemensPerMeter):
        """
        Get a ElectricConductivity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricConductivity unit to store the ElectricConductivity value in the DTO representation.
        :type hold_in_unit: ElectricConductivityUnits
        :return: JSON object represents ElectricConductivity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "SiemensPerMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_conductivity_dto: ElectricConductivityDto):
        """
        Obtain a new instance of ElectricConductivity from a DTO unit object.

        :param electric_conductivity_dto: The ElectricConductivity DTO representation.
        :type electric_conductivity_dto: ElectricConductivityDto
        :return: A new instance of ElectricConductivity.
        :rtype: ElectricConductivity
        """
        return ElectricConductivity(electric_conductivity_dto.value, electric_conductivity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricConductivity from a DTO unit json representation.

        :param data: The ElectricConductivity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "SiemensPerMeter"}
        :return: A new instance of ElectricConductivity.
        :rtype: ElectricConductivity
        """
        return ElectricConductivity.from_dto(ElectricConductivityDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricConductivityUnits) -> float:
        value = self._value
        
        if from_unit == ElectricConductivityUnits.SiemensPerMeter:
            return (value)
        
        if from_unit == ElectricConductivityUnits.SiemensPerInch:
            return (value / 3.937007874015748e1)
        
        if from_unit == ElectricConductivityUnits.SiemensPerFoot:
            return (value / 3.2808398950131234)
        
        if from_unit == ElectricConductivityUnits.SiemensPerCentimeter:
            return (value / 1e2)
        
        if from_unit == ElectricConductivityUnits.MicrosiemensPerCentimeter:
            return ((value / 1e2) / 1e-06)
        
        if from_unit == ElectricConductivityUnits.MillisiemensPerCentimeter:
            return ((value / 1e2) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricConductivityUnits) -> float:
        
        if to_unit == ElectricConductivityUnits.SiemensPerMeter:
            return (value)
        
        if to_unit == ElectricConductivityUnits.SiemensPerInch:
            return (value * 3.937007874015748e1)
        
        if to_unit == ElectricConductivityUnits.SiemensPerFoot:
            return (value * 3.2808398950131234)
        
        if to_unit == ElectricConductivityUnits.SiemensPerCentimeter:
            return (value * 1e2)
        
        if to_unit == ElectricConductivityUnits.MicrosiemensPerCentimeter:
            return ((value * 1e2) * 1e-06)
        
        if to_unit == ElectricConductivityUnits.MillisiemensPerCentimeter:
            return ((value * 1e2) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_siemens_per_meter(siemens_per_meter: float):
        """
        Create a new instance of ElectricConductivity from a value in siemens_per_meter.

        

        :param meters: The ElectricConductivity value in siemens_per_meter.
        :type siemens_per_meter: float
        :return: A new instance of ElectricConductivity.
        :rtype: ElectricConductivity
        """
        return ElectricConductivity(siemens_per_meter, ElectricConductivityUnits.SiemensPerMeter)

    
    @staticmethod
    def from_siemens_per_inch(siemens_per_inch: float):
        """
        Create a new instance of ElectricConductivity from a value in siemens_per_inch.

        

        :param meters: The ElectricConductivity value in siemens_per_inch.
        :type siemens_per_inch: float
        :return: A new instance of ElectricConductivity.
        :rtype: ElectricConductivity
        """
        return ElectricConductivity(siemens_per_inch, ElectricConductivityUnits.SiemensPerInch)

    
    @staticmethod
    def from_siemens_per_foot(siemens_per_foot: float):
        """
        Create a new instance of ElectricConductivity from a value in siemens_per_foot.

        

        :param meters: The ElectricConductivity value in siemens_per_foot.
        :type siemens_per_foot: float
        :return: A new instance of ElectricConductivity.
        :rtype: ElectricConductivity
        """
        return ElectricConductivity(siemens_per_foot, ElectricConductivityUnits.SiemensPerFoot)

    
    @staticmethod
    def from_siemens_per_centimeter(siemens_per_centimeter: float):
        """
        Create a new instance of ElectricConductivity from a value in siemens_per_centimeter.

        

        :param meters: The ElectricConductivity value in siemens_per_centimeter.
        :type siemens_per_centimeter: float
        :return: A new instance of ElectricConductivity.
        :rtype: ElectricConductivity
        """
        return ElectricConductivity(siemens_per_centimeter, ElectricConductivityUnits.SiemensPerCentimeter)

    
    @staticmethod
    def from_microsiemens_per_centimeter(microsiemens_per_centimeter: float):
        """
        Create a new instance of ElectricConductivity from a value in microsiemens_per_centimeter.

        

        :param meters: The ElectricConductivity value in microsiemens_per_centimeter.
        :type microsiemens_per_centimeter: float
        :return: A new instance of ElectricConductivity.
        :rtype: ElectricConductivity
        """
        return ElectricConductivity(microsiemens_per_centimeter, ElectricConductivityUnits.MicrosiemensPerCentimeter)

    
    @staticmethod
    def from_millisiemens_per_centimeter(millisiemens_per_centimeter: float):
        """
        Create a new instance of ElectricConductivity from a value in millisiemens_per_centimeter.

        

        :param meters: The ElectricConductivity value in millisiemens_per_centimeter.
        :type millisiemens_per_centimeter: float
        :return: A new instance of ElectricConductivity.
        :rtype: ElectricConductivity
        """
        return ElectricConductivity(millisiemens_per_centimeter, ElectricConductivityUnits.MillisiemensPerCentimeter)

    
    @property
    def siemens_per_meter(self) -> float:
        """
        
        """
        if self.__siemens_per_meter != None:
            return self.__siemens_per_meter
        self.__siemens_per_meter = self.__convert_from_base(ElectricConductivityUnits.SiemensPerMeter)
        return self.__siemens_per_meter

    
    @property
    def siemens_per_inch(self) -> float:
        """
        
        """
        if self.__siemens_per_inch != None:
            return self.__siemens_per_inch
        self.__siemens_per_inch = self.__convert_from_base(ElectricConductivityUnits.SiemensPerInch)
        return self.__siemens_per_inch

    
    @property
    def siemens_per_foot(self) -> float:
        """
        
        """
        if self.__siemens_per_foot != None:
            return self.__siemens_per_foot
        self.__siemens_per_foot = self.__convert_from_base(ElectricConductivityUnits.SiemensPerFoot)
        return self.__siemens_per_foot

    
    @property
    def siemens_per_centimeter(self) -> float:
        """
        
        """
        if self.__siemens_per_centimeter != None:
            return self.__siemens_per_centimeter
        self.__siemens_per_centimeter = self.__convert_from_base(ElectricConductivityUnits.SiemensPerCentimeter)
        return self.__siemens_per_centimeter

    
    @property
    def microsiemens_per_centimeter(self) -> float:
        """
        
        """
        if self.__microsiemens_per_centimeter != None:
            return self.__microsiemens_per_centimeter
        self.__microsiemens_per_centimeter = self.__convert_from_base(ElectricConductivityUnits.MicrosiemensPerCentimeter)
        return self.__microsiemens_per_centimeter

    
    @property
    def millisiemens_per_centimeter(self) -> float:
        """
        
        """
        if self.__millisiemens_per_centimeter != None:
            return self.__millisiemens_per_centimeter
        self.__millisiemens_per_centimeter = self.__convert_from_base(ElectricConductivityUnits.MillisiemensPerCentimeter)
        return self.__millisiemens_per_centimeter

    
    def to_string(self, unit: ElectricConductivityUnits = ElectricConductivityUnits.SiemensPerMeter, fractional_digits: int = None) -> str:
        """
        Format the ElectricConductivity to a string.
        
        Note: the default format for ElectricConductivity is SiemensPerMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricConductivity. Default is 'SiemensPerMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricConductivityUnits.SiemensPerMeter:
            return f"""{super()._truncate_fraction_digits(self.siemens_per_meter, fractional_digits)} S/m"""
        
        if unit == ElectricConductivityUnits.SiemensPerInch:
            return f"""{super()._truncate_fraction_digits(self.siemens_per_inch, fractional_digits)} S/in"""
        
        if unit == ElectricConductivityUnits.SiemensPerFoot:
            return f"""{super()._truncate_fraction_digits(self.siemens_per_foot, fractional_digits)} S/ft"""
        
        if unit == ElectricConductivityUnits.SiemensPerCentimeter:
            return f"""{super()._truncate_fraction_digits(self.siemens_per_centimeter, fractional_digits)} S/cm"""
        
        if unit == ElectricConductivityUnits.MicrosiemensPerCentimeter:
            return f"""{super()._truncate_fraction_digits(self.microsiemens_per_centimeter, fractional_digits)} μS/cm"""
        
        if unit == ElectricConductivityUnits.MillisiemensPerCentimeter:
            return f"""{super()._truncate_fraction_digits(self.millisiemens_per_centimeter, fractional_digits)} mS/cm"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricConductivityUnits = ElectricConductivityUnits.SiemensPerMeter) -> str:
        """
        Get ElectricConductivity unit abbreviation.
        Note! the default abbreviation for ElectricConductivity is SiemensPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricConductivityUnits.SiemensPerMeter:
            return """S/m"""
        
        if unit_abbreviation == ElectricConductivityUnits.SiemensPerInch:
            return """S/in"""
        
        if unit_abbreviation == ElectricConductivityUnits.SiemensPerFoot:
            return """S/ft"""
        
        if unit_abbreviation == ElectricConductivityUnits.SiemensPerCentimeter:
            return """S/cm"""
        
        if unit_abbreviation == ElectricConductivityUnits.MicrosiemensPerCentimeter:
            return """μS/cm"""
        
        if unit_abbreviation == ElectricConductivityUnits.MillisiemensPerCentimeter:
            return """mS/cm"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class AbsorbedDoseOfIonizingRadiationUnits(Enum):
        """
            AbsorbedDoseOfIonizingRadiationUnits enumeration
        """
        
        Gray = 'Gray'
        """
            The gray is the unit of ionizing radiation dose in the SI, defined as the absorption of one joule of radiation energy per kilogram of matter.
        """
        
        Rad = 'Rad'
        """
            The rad is a unit of absorbed radiation dose, defined as 1 rad = 0.01 Gy = 0.01 J/kg.
        """
        
        Femtogray = 'Femtogray'
        """
            
        """
        
        Picogray = 'Picogray'
        """
            
        """
        
        Nanogray = 'Nanogray'
        """
            
        """
        
        Microgray = 'Microgray'
        """
            
        """
        
        Milligray = 'Milligray'
        """
            
        """
        
        Centigray = 'Centigray'
        """
            
        """
        
        Kilogray = 'Kilogray'
        """
            
        """
        
        Megagray = 'Megagray'
        """
            
        """
        
        Gigagray = 'Gigagray'
        """
            
        """
        
        Teragray = 'Teragray'
        """
            
        """
        
        Petagray = 'Petagray'
        """
            
        """
        
        Millirad = 'Millirad'
        """
            
        """
        
        Kilorad = 'Kilorad'
        """
            
        """
        
        Megarad = 'Megarad'
        """
            
        """
        

class AbsorbedDoseOfIonizingRadiationDto:
    """
    A DTO representation of a AbsorbedDoseOfIonizingRadiation

    Attributes:
        value (float): The value of the AbsorbedDoseOfIonizingRadiation.
        unit (AbsorbedDoseOfIonizingRadiationUnits): The specific unit that the AbsorbedDoseOfIonizingRadiation value is representing.
    """

    def __init__(self, value: float, unit: AbsorbedDoseOfIonizingRadiationUnits):
        """
        Create a new DTO representation of a AbsorbedDoseOfIonizingRadiation

        Parameters:
            value (float): The value of the AbsorbedDoseOfIonizingRadiation.
            unit (AbsorbedDoseOfIonizingRadiationUnits): The specific unit that the AbsorbedDoseOfIonizingRadiation value is representing.
        """
        self.value: float = value
        """
        The value of the AbsorbedDoseOfIonizingRadiation
        """
        self.unit: AbsorbedDoseOfIonizingRadiationUnits = unit
        """
        The specific unit that the AbsorbedDoseOfIonizingRadiation value is representing
        """

    def to_json(self):
        """
        Get a AbsorbedDoseOfIonizingRadiation DTO JSON object representing the current unit.

        :return: JSON object represents AbsorbedDoseOfIonizingRadiation DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Gray"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of AbsorbedDoseOfIonizingRadiation DTO from a json representation.

        :param data: The AbsorbedDoseOfIonizingRadiation DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Gray"}
        :return: A new instance of AbsorbedDoseOfIonizingRadiationDto.
        :rtype: AbsorbedDoseOfIonizingRadiationDto
        """
        return AbsorbedDoseOfIonizingRadiationDto(value=data["value"], unit=AbsorbedDoseOfIonizingRadiationUnits(data["unit"]))


class AbsorbedDoseOfIonizingRadiation(AbstractMeasure):
    """
    Absorbed dose is a dose quantity which is the measure of the energy deposited in matter by ionizing radiation per unit mass.

    Args:
        value (float): The value.
        from_unit (AbsorbedDoseOfIonizingRadiationUnits): The AbsorbedDoseOfIonizingRadiation unit to create from, The default unit is Gray
    """
    def __init__(self, value: float, from_unit: AbsorbedDoseOfIonizingRadiationUnits = AbsorbedDoseOfIonizingRadiationUnits.Gray):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__grays = None
        
        self.__rads = None
        
        self.__femtograys = None
        
        self.__picograys = None
        
        self.__nanograys = None
        
        self.__micrograys = None
        
        self.__milligrays = None
        
        self.__centigrays = None
        
        self.__kilograys = None
        
        self.__megagrays = None
        
        self.__gigagrays = None
        
        self.__teragrays = None
        
        self.__petagrays = None
        
        self.__millirads = None
        
        self.__kilorads = None
        
        self.__megarads = None
        

    def convert(self, unit: AbsorbedDoseOfIonizingRadiationUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: AbsorbedDoseOfIonizingRadiationUnits = AbsorbedDoseOfIonizingRadiationUnits.Gray) -> AbsorbedDoseOfIonizingRadiationDto:
        """
        Get a new instance of AbsorbedDoseOfIonizingRadiation DTO representing the current unit.

        :param hold_in_unit: The specific AbsorbedDoseOfIonizingRadiation unit to store the AbsorbedDoseOfIonizingRadiation value in the DTO representation.
        :type hold_in_unit: AbsorbedDoseOfIonizingRadiationUnits
        :return: A new instance of AbsorbedDoseOfIonizingRadiationDto.
        :rtype: AbsorbedDoseOfIonizingRadiationDto
        """
        return AbsorbedDoseOfIonizingRadiationDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: AbsorbedDoseOfIonizingRadiationUnits = AbsorbedDoseOfIonizingRadiationUnits.Gray):
        """
        Get a AbsorbedDoseOfIonizingRadiation DTO JSON object representing the current unit.

        :param hold_in_unit: The specific AbsorbedDoseOfIonizingRadiation unit to store the AbsorbedDoseOfIonizingRadiation value in the DTO representation.
        :type hold_in_unit: AbsorbedDoseOfIonizingRadiationUnits
        :return: JSON object represents AbsorbedDoseOfIonizingRadiation DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Gray"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(absorbed_dose_of_ionizing_radiation_dto: AbsorbedDoseOfIonizingRadiationDto):
        """
        Obtain a new instance of AbsorbedDoseOfIonizingRadiation from a DTO unit object.

        :param absorbed_dose_of_ionizing_radiation_dto: The AbsorbedDoseOfIonizingRadiation DTO representation.
        :type absorbed_dose_of_ionizing_radiation_dto: AbsorbedDoseOfIonizingRadiationDto
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(absorbed_dose_of_ionizing_radiation_dto.value, absorbed_dose_of_ionizing_radiation_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of AbsorbedDoseOfIonizingRadiation from a DTO unit json representation.

        :param data: The AbsorbedDoseOfIonizingRadiation DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Gray"}
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation.from_dto(AbsorbedDoseOfIonizingRadiationDto.from_json(data))

    def __convert_from_base(self, from_unit: AbsorbedDoseOfIonizingRadiationUnits) -> float:
        value = self._value
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Gray:
            return (value)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Rad:
            return (value * 100)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Femtogray:
            return ((value) / 1e-15)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Picogray:
            return ((value) / 1e-12)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Nanogray:
            return ((value) / 1e-09)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Microgray:
            return ((value) / 1e-06)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Milligray:
            return ((value) / 0.001)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Centigray:
            return ((value) / 0.01)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Kilogray:
            return ((value) / 1000.0)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Megagray:
            return ((value) / 1000000.0)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Gigagray:
            return ((value) / 1000000000.0)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Teragray:
            return ((value) / 1000000000000.0)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Petagray:
            return ((value) / 1000000000000000.0)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Millirad:
            return ((value * 100) / 0.001)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Kilorad:
            return ((value * 100) / 1000.0)
        
        if from_unit == AbsorbedDoseOfIonizingRadiationUnits.Megarad:
            return ((value * 100) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: AbsorbedDoseOfIonizingRadiationUnits) -> float:
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Gray:
            return (value)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Rad:
            return (value / 100)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Femtogray:
            return ((value) * 1e-15)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Picogray:
            return ((value) * 1e-12)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Nanogray:
            return ((value) * 1e-09)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Microgray:
            return ((value) * 1e-06)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Milligray:
            return ((value) * 0.001)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Centigray:
            return ((value) * 0.01)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Kilogray:
            return ((value) * 1000.0)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Megagray:
            return ((value) * 1000000.0)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Gigagray:
            return ((value) * 1000000000.0)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Teragray:
            return ((value) * 1000000000000.0)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Petagray:
            return ((value) * 1000000000000000.0)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Millirad:
            return ((value / 100) * 0.001)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Kilorad:
            return ((value / 100) * 1000.0)
        
        if to_unit == AbsorbedDoseOfIonizingRadiationUnits.Megarad:
            return ((value / 100) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_grays(grays: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in grays.

        The gray is the unit of ionizing radiation dose in the SI, defined as the absorption of one joule of radiation energy per kilogram of matter.

        :param meters: The AbsorbedDoseOfIonizingRadiation value in grays.
        :type grays: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(grays, AbsorbedDoseOfIonizingRadiationUnits.Gray)

    
    @staticmethod
    def from_rads(rads: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in rads.

        The rad is a unit of absorbed radiation dose, defined as 1 rad = 0.01 Gy = 0.01 J/kg.

        :param meters: The AbsorbedDoseOfIonizingRadiation value in rads.
        :type rads: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(rads, AbsorbedDoseOfIonizingRadiationUnits.Rad)

    
    @staticmethod
    def from_femtograys(femtograys: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in femtograys.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in femtograys.
        :type femtograys: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(femtograys, AbsorbedDoseOfIonizingRadiationUnits.Femtogray)

    
    @staticmethod
    def from_picograys(picograys: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in picograys.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in picograys.
        :type picograys: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(picograys, AbsorbedDoseOfIonizingRadiationUnits.Picogray)

    
    @staticmethod
    def from_nanograys(nanograys: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in nanograys.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in nanograys.
        :type nanograys: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(nanograys, AbsorbedDoseOfIonizingRadiationUnits.Nanogray)

    
    @staticmethod
    def from_micrograys(micrograys: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in micrograys.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in micrograys.
        :type micrograys: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(micrograys, AbsorbedDoseOfIonizingRadiationUnits.Microgray)

    
    @staticmethod
    def from_milligrays(milligrays: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in milligrays.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in milligrays.
        :type milligrays: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(milligrays, AbsorbedDoseOfIonizingRadiationUnits.Milligray)

    
    @staticmethod
    def from_centigrays(centigrays: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in centigrays.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in centigrays.
        :type centigrays: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(centigrays, AbsorbedDoseOfIonizingRadiationUnits.Centigray)

    
    @staticmethod
    def from_kilograys(kilograys: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in kilograys.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in kilograys.
        :type kilograys: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(kilograys, AbsorbedDoseOfIonizingRadiationUnits.Kilogray)

    
    @staticmethod
    def from_megagrays(megagrays: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in megagrays.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in megagrays.
        :type megagrays: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(megagrays, AbsorbedDoseOfIonizingRadiationUnits.Megagray)

    
    @staticmethod
    def from_gigagrays(gigagrays: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in gigagrays.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in gigagrays.
        :type gigagrays: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(gigagrays, AbsorbedDoseOfIonizingRadiationUnits.Gigagray)

    
    @staticmethod
    def from_teragrays(teragrays: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in teragrays.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in teragrays.
        :type teragrays: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(teragrays, AbsorbedDoseOfIonizingRadiationUnits.Teragray)

    
    @staticmethod
    def from_petagrays(petagrays: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in petagrays.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in petagrays.
        :type petagrays: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(petagrays, AbsorbedDoseOfIonizingRadiationUnits.Petagray)

    
    @staticmethod
    def from_millirads(millirads: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in millirads.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in millirads.
        :type millirads: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(millirads, AbsorbedDoseOfIonizingRadiationUnits.Millirad)

    
    @staticmethod
    def from_kilorads(kilorads: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in kilorads.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in kilorads.
        :type kilorads: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(kilorads, AbsorbedDoseOfIonizingRadiationUnits.Kilorad)

    
    @staticmethod
    def from_megarads(megarads: float):
        """
        Create a new instance of AbsorbedDoseOfIonizingRadiation from a value in megarads.

        

        :param meters: The AbsorbedDoseOfIonizingRadiation value in megarads.
        :type megarads: float
        :return: A new instance of AbsorbedDoseOfIonizingRadiation.
        :rtype: AbsorbedDoseOfIonizingRadiation
        """
        return AbsorbedDoseOfIonizingRadiation(megarads, AbsorbedDoseOfIonizingRadiationUnits.Megarad)

    
    @property
    def grays(self) -> float:
        """
        The gray is the unit of ionizing radiation dose in the SI, defined as the absorption of one joule of radiation energy per kilogram of matter.
        """
        if self.__grays != None:
            return self.__grays
        self.__grays = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Gray)
        return self.__grays

    
    @property
    def rads(self) -> float:
        """
        The rad is a unit of absorbed radiation dose, defined as 1 rad = 0.01 Gy = 0.01 J/kg.
        """
        if self.__rads != None:
            return self.__rads
        self.__rads = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Rad)
        return self.__rads

    
    @property
    def femtograys(self) -> float:
        """
        
        """
        if self.__femtograys != None:
            return self.__femtograys
        self.__femtograys = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Femtogray)
        return self.__femtograys

    
    @property
    def picograys(self) -> float:
        """
        
        """
        if self.__picograys != None:
            return self.__picograys
        self.__picograys = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Picogray)
        return self.__picograys

    
    @property
    def nanograys(self) -> float:
        """
        
        """
        if self.__nanograys != None:
            return self.__nanograys
        self.__nanograys = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Nanogray)
        return self.__nanograys

    
    @property
    def micrograys(self) -> float:
        """
        
        """
        if self.__micrograys != None:
            return self.__micrograys
        self.__micrograys = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Microgray)
        return self.__micrograys

    
    @property
    def milligrays(self) -> float:
        """
        
        """
        if self.__milligrays != None:
            return self.__milligrays
        self.__milligrays = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Milligray)
        return self.__milligrays

    
    @property
    def centigrays(self) -> float:
        """
        
        """
        if self.__centigrays != None:
            return self.__centigrays
        self.__centigrays = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Centigray)
        return self.__centigrays

    
    @property
    def kilograys(self) -> float:
        """
        
        """
        if self.__kilograys != None:
            return self.__kilograys
        self.__kilograys = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Kilogray)
        return self.__kilograys

    
    @property
    def megagrays(self) -> float:
        """
        
        """
        if self.__megagrays != None:
            return self.__megagrays
        self.__megagrays = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Megagray)
        return self.__megagrays

    
    @property
    def gigagrays(self) -> float:
        """
        
        """
        if self.__gigagrays != None:
            return self.__gigagrays
        self.__gigagrays = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Gigagray)
        return self.__gigagrays

    
    @property
    def teragrays(self) -> float:
        """
        
        """
        if self.__teragrays != None:
            return self.__teragrays
        self.__teragrays = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Teragray)
        return self.__teragrays

    
    @property
    def petagrays(self) -> float:
        """
        
        """
        if self.__petagrays != None:
            return self.__petagrays
        self.__petagrays = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Petagray)
        return self.__petagrays

    
    @property
    def millirads(self) -> float:
        """
        
        """
        if self.__millirads != None:
            return self.__millirads
        self.__millirads = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Millirad)
        return self.__millirads

    
    @property
    def kilorads(self) -> float:
        """
        
        """
        if self.__kilorads != None:
            return self.__kilorads
        self.__kilorads = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Kilorad)
        return self.__kilorads

    
    @property
    def megarads(self) -> float:
        """
        
        """
        if self.__megarads != None:
            return self.__megarads
        self.__megarads = self.__convert_from_base(AbsorbedDoseOfIonizingRadiationUnits.Megarad)
        return self.__megarads

    
    def to_string(self, unit: AbsorbedDoseOfIonizingRadiationUnits = AbsorbedDoseOfIonizingRadiationUnits.Gray, fractional_digits: int = None) -> str:
        """
        Format the AbsorbedDoseOfIonizingRadiation to a string.
        
        Note: the default format for AbsorbedDoseOfIonizingRadiation is Gray.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the AbsorbedDoseOfIonizingRadiation. Default is 'Gray'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Gray:
            return f"""{super()._truncate_fraction_digits(self.grays, fractional_digits)} Gy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Rad:
            return f"""{super()._truncate_fraction_digits(self.rads, fractional_digits)} rad"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Femtogray:
            return f"""{super()._truncate_fraction_digits(self.femtograys, fractional_digits)} fGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Picogray:
            return f"""{super()._truncate_fraction_digits(self.picograys, fractional_digits)} pGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Nanogray:
            return f"""{super()._truncate_fraction_digits(self.nanograys, fractional_digits)} nGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Microgray:
            return f"""{super()._truncate_fraction_digits(self.micrograys, fractional_digits)} μGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Milligray:
            return f"""{super()._truncate_fraction_digits(self.milligrays, fractional_digits)} mGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Centigray:
            return f"""{super()._truncate_fraction_digits(self.centigrays, fractional_digits)} cGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Kilogray:
            return f"""{super()._truncate_fraction_digits(self.kilograys, fractional_digits)} kGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Megagray:
            return f"""{super()._truncate_fraction_digits(self.megagrays, fractional_digits)} MGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Gigagray:
            return f"""{super()._truncate_fraction_digits(self.gigagrays, fractional_digits)} GGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Teragray:
            return f"""{super()._truncate_fraction_digits(self.teragrays, fractional_digits)} TGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Petagray:
            return f"""{super()._truncate_fraction_digits(self.petagrays, fractional_digits)} PGy"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Millirad:
            return f"""{super()._truncate_fraction_digits(self.millirads, fractional_digits)} mrad"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Kilorad:
            return f"""{super()._truncate_fraction_digits(self.kilorads, fractional_digits)} krad"""
        
        if unit == AbsorbedDoseOfIonizingRadiationUnits.Megarad:
            return f"""{super()._truncate_fraction_digits(self.megarads, fractional_digits)} Mrad"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: AbsorbedDoseOfIonizingRadiationUnits = AbsorbedDoseOfIonizingRadiationUnits.Gray) -> str:
        """
        Get AbsorbedDoseOfIonizingRadiation unit abbreviation.
        Note! the default abbreviation for AbsorbedDoseOfIonizingRadiation is Gray.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Gray:
            return """Gy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Rad:
            return """rad"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Femtogray:
            return """fGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Picogray:
            return """pGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Nanogray:
            return """nGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Microgray:
            return """μGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Milligray:
            return """mGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Centigray:
            return """cGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Kilogray:
            return """kGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Megagray:
            return """MGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Gigagray:
            return """GGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Teragray:
            return """TGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Petagray:
            return """PGy"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Millirad:
            return """mrad"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Kilorad:
            return """krad"""
        
        if unit_abbreviation == AbsorbedDoseOfIonizingRadiationUnits.Megarad:
            return """Mrad"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class SpecificVolumeUnits(Enum):
        """
            SpecificVolumeUnits enumeration
        """
        
        CubicMeterPerKilogram = 'CubicMeterPerKilogram'
        """
            
        """
        
        CubicFootPerPound = 'CubicFootPerPound'
        """
            
        """
        
        MillicubicMeterPerKilogram = 'MillicubicMeterPerKilogram'
        """
            
        """
        

class SpecificVolumeDto:
    """
    A DTO representation of a SpecificVolume

    Attributes:
        value (float): The value of the SpecificVolume.
        unit (SpecificVolumeUnits): The specific unit that the SpecificVolume value is representing.
    """

    def __init__(self, value: float, unit: SpecificVolumeUnits):
        """
        Create a new DTO representation of a SpecificVolume

        Parameters:
            value (float): The value of the SpecificVolume.
            unit (SpecificVolumeUnits): The specific unit that the SpecificVolume value is representing.
        """
        self.value: float = value
        """
        The value of the SpecificVolume
        """
        self.unit: SpecificVolumeUnits = unit
        """
        The specific unit that the SpecificVolume value is representing
        """

    def to_json(self):
        """
        Get a SpecificVolume DTO JSON object representing the current unit.

        :return: JSON object represents SpecificVolume DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CubicMeterPerKilogram"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of SpecificVolume DTO from a json representation.

        :param data: The SpecificVolume DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CubicMeterPerKilogram"}
        :return: A new instance of SpecificVolumeDto.
        :rtype: SpecificVolumeDto
        """
        return SpecificVolumeDto(value=data["value"], unit=SpecificVolumeUnits(data["unit"]))


class SpecificVolume(AbstractMeasure):
    """
    In thermodynamics, the specific volume of a substance is the ratio of the substance's volume to its mass. It is the reciprocal of density and an intrinsic property of matter as well.

    Args:
        value (float): The value.
        from_unit (SpecificVolumeUnits): The SpecificVolume unit to create from, The default unit is CubicMeterPerKilogram
    """
    def __init__(self, value: float, from_unit: SpecificVolumeUnits = SpecificVolumeUnits.CubicMeterPerKilogram):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__cubic_meters_per_kilogram = None
        
        self.__cubic_feet_per_pound = None
        
        self.__millicubic_meters_per_kilogram = None
        

    def convert(self, unit: SpecificVolumeUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: SpecificVolumeUnits = SpecificVolumeUnits.CubicMeterPerKilogram) -> SpecificVolumeDto:
        """
        Get a new instance of SpecificVolume DTO representing the current unit.

        :param hold_in_unit: The specific SpecificVolume unit to store the SpecificVolume value in the DTO representation.
        :type hold_in_unit: SpecificVolumeUnits
        :return: A new instance of SpecificVolumeDto.
        :rtype: SpecificVolumeDto
        """
        return SpecificVolumeDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: SpecificVolumeUnits = SpecificVolumeUnits.CubicMeterPerKilogram):
        """
        Get a SpecificVolume DTO JSON object representing the current unit.

        :param hold_in_unit: The specific SpecificVolume unit to store the SpecificVolume value in the DTO representation.
        :type hold_in_unit: SpecificVolumeUnits
        :return: JSON object represents SpecificVolume DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CubicMeterPerKilogram"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(specific_volume_dto: SpecificVolumeDto):
        """
        Obtain a new instance of SpecificVolume from a DTO unit object.

        :param specific_volume_dto: The SpecificVolume DTO representation.
        :type specific_volume_dto: SpecificVolumeDto
        :return: A new instance of SpecificVolume.
        :rtype: SpecificVolume
        """
        return SpecificVolume(specific_volume_dto.value, specific_volume_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of SpecificVolume from a DTO unit json representation.

        :param data: The SpecificVolume DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CubicMeterPerKilogram"}
        :return: A new instance of SpecificVolume.
        :rtype: SpecificVolume
        """
        return SpecificVolume.from_dto(SpecificVolumeDto.from_json(data))

    def __convert_from_base(self, from_unit: SpecificVolumeUnits) -> float:
        value = self._value
        
        if from_unit == SpecificVolumeUnits.CubicMeterPerKilogram:
            return (value)
        
        if from_unit == SpecificVolumeUnits.CubicFootPerPound:
            return (value * 16.01846353)
        
        if from_unit == SpecificVolumeUnits.MillicubicMeterPerKilogram:
            return ((value) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: SpecificVolumeUnits) -> float:
        
        if to_unit == SpecificVolumeUnits.CubicMeterPerKilogram:
            return (value)
        
        if to_unit == SpecificVolumeUnits.CubicFootPerPound:
            return (value / 16.01846353)
        
        if to_unit == SpecificVolumeUnits.MillicubicMeterPerKilogram:
            return ((value) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_cubic_meters_per_kilogram(cubic_meters_per_kilogram: float):
        """
        Create a new instance of SpecificVolume from a value in cubic_meters_per_kilogram.

        

        :param meters: The SpecificVolume value in cubic_meters_per_kilogram.
        :type cubic_meters_per_kilogram: float
        :return: A new instance of SpecificVolume.
        :rtype: SpecificVolume
        """
        return SpecificVolume(cubic_meters_per_kilogram, SpecificVolumeUnits.CubicMeterPerKilogram)

    
    @staticmethod
    def from_cubic_feet_per_pound(cubic_feet_per_pound: float):
        """
        Create a new instance of SpecificVolume from a value in cubic_feet_per_pound.

        

        :param meters: The SpecificVolume value in cubic_feet_per_pound.
        :type cubic_feet_per_pound: float
        :return: A new instance of SpecificVolume.
        :rtype: SpecificVolume
        """
        return SpecificVolume(cubic_feet_per_pound, SpecificVolumeUnits.CubicFootPerPound)

    
    @staticmethod
    def from_millicubic_meters_per_kilogram(millicubic_meters_per_kilogram: float):
        """
        Create a new instance of SpecificVolume from a value in millicubic_meters_per_kilogram.

        

        :param meters: The SpecificVolume value in millicubic_meters_per_kilogram.
        :type millicubic_meters_per_kilogram: float
        :return: A new instance of SpecificVolume.
        :rtype: SpecificVolume
        """
        return SpecificVolume(millicubic_meters_per_kilogram, SpecificVolumeUnits.MillicubicMeterPerKilogram)

    
    @property
    def cubic_meters_per_kilogram(self) -> float:
        """
        
        """
        if self.__cubic_meters_per_kilogram != None:
            return self.__cubic_meters_per_kilogram
        self.__cubic_meters_per_kilogram = self.__convert_from_base(SpecificVolumeUnits.CubicMeterPerKilogram)
        return self.__cubic_meters_per_kilogram

    
    @property
    def cubic_feet_per_pound(self) -> float:
        """
        
        """
        if self.__cubic_feet_per_pound != None:
            return self.__cubic_feet_per_pound
        self.__cubic_feet_per_pound = self.__convert_from_base(SpecificVolumeUnits.CubicFootPerPound)
        return self.__cubic_feet_per_pound

    
    @property
    def millicubic_meters_per_kilogram(self) -> float:
        """
        
        """
        if self.__millicubic_meters_per_kilogram != None:
            return self.__millicubic_meters_per_kilogram
        self.__millicubic_meters_per_kilogram = self.__convert_from_base(SpecificVolumeUnits.MillicubicMeterPerKilogram)
        return self.__millicubic_meters_per_kilogram

    
    def to_string(self, unit: SpecificVolumeUnits = SpecificVolumeUnits.CubicMeterPerKilogram, fractional_digits: int = None) -> str:
        """
        Format the SpecificVolume to a string.
        
        Note: the default format for SpecificVolume is CubicMeterPerKilogram.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the SpecificVolume. Default is 'CubicMeterPerKilogram'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == SpecificVolumeUnits.CubicMeterPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.cubic_meters_per_kilogram, fractional_digits)} m³/kg"""
        
        if unit == SpecificVolumeUnits.CubicFootPerPound:
            return f"""{super()._truncate_fraction_digits(self.cubic_feet_per_pound, fractional_digits)} ft³/lb"""
        
        if unit == SpecificVolumeUnits.MillicubicMeterPerKilogram:
            return f"""{super()._truncate_fraction_digits(self.millicubic_meters_per_kilogram, fractional_digits)} mm³/kg"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: SpecificVolumeUnits = SpecificVolumeUnits.CubicMeterPerKilogram) -> str:
        """
        Get SpecificVolume unit abbreviation.
        Note! the default abbreviation for SpecificVolume is CubicMeterPerKilogram.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == SpecificVolumeUnits.CubicMeterPerKilogram:
            return """m³/kg"""
        
        if unit_abbreviation == SpecificVolumeUnits.CubicFootPerPound:
            return """ft³/lb"""
        
        if unit_abbreviation == SpecificVolumeUnits.MillicubicMeterPerKilogram:
            return """mm³/kg"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class TurbidityUnits(Enum):
        """
            TurbidityUnits enumeration
        """
        
        NTU = 'NTU'
        """
            
        """
        

class TurbidityDto:
    """
    A DTO representation of a Turbidity

    Attributes:
        value (float): The value of the Turbidity.
        unit (TurbidityUnits): The specific unit that the Turbidity value is representing.
    """

    def __init__(self, value: float, unit: TurbidityUnits):
        """
        Create a new DTO representation of a Turbidity

        Parameters:
            value (float): The value of the Turbidity.
            unit (TurbidityUnits): The specific unit that the Turbidity value is representing.
        """
        self.value: float = value
        """
        The value of the Turbidity
        """
        self.unit: TurbidityUnits = unit
        """
        The specific unit that the Turbidity value is representing
        """

    def to_json(self):
        """
        Get a Turbidity DTO JSON object representing the current unit.

        :return: JSON object represents Turbidity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NTU"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Turbidity DTO from a json representation.

        :param data: The Turbidity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NTU"}
        :return: A new instance of TurbidityDto.
        :rtype: TurbidityDto
        """
        return TurbidityDto(value=data["value"], unit=TurbidityUnits(data["unit"]))


class Turbidity(AbstractMeasure):
    """
    Turbidity is the cloudiness or haziness of a fluid caused by large numbers of individual particles that are generally invisible to the naked eye, similar to smoke in air. The measurement of turbidity is a key test of water quality.

    Args:
        value (float): The value.
        from_unit (TurbidityUnits): The Turbidity unit to create from, The default unit is NTU
    """
    def __init__(self, value: float, from_unit: TurbidityUnits = TurbidityUnits.NTU):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__ntu = None
        

    def convert(self, unit: TurbidityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: TurbidityUnits = TurbidityUnits.NTU) -> TurbidityDto:
        """
        Get a new instance of Turbidity DTO representing the current unit.

        :param hold_in_unit: The specific Turbidity unit to store the Turbidity value in the DTO representation.
        :type hold_in_unit: TurbidityUnits
        :return: A new instance of TurbidityDto.
        :rtype: TurbidityDto
        """
        return TurbidityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: TurbidityUnits = TurbidityUnits.NTU):
        """
        Get a Turbidity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Turbidity unit to store the Turbidity value in the DTO representation.
        :type hold_in_unit: TurbidityUnits
        :return: JSON object represents Turbidity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NTU"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(turbidity_dto: TurbidityDto):
        """
        Obtain a new instance of Turbidity from a DTO unit object.

        :param turbidity_dto: The Turbidity DTO representation.
        :type turbidity_dto: TurbidityDto
        :return: A new instance of Turbidity.
        :rtype: Turbidity
        """
        return Turbidity(turbidity_dto.value, turbidity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Turbidity from a DTO unit json representation.

        :param data: The Turbidity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NTU"}
        :return: A new instance of Turbidity.
        :rtype: Turbidity
        """
        return Turbidity.from_dto(TurbidityDto.from_json(data))

    def __convert_from_base(self, from_unit: TurbidityUnits) -> float:
        value = self._value
        
        if from_unit == TurbidityUnits.NTU:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TurbidityUnits) -> float:
        
        if to_unit == TurbidityUnits.NTU:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_ntu(ntu: float):
        """
        Create a new instance of Turbidity from a value in ntu.

        

        :param meters: The Turbidity value in ntu.
        :type ntu: float
        :return: A new instance of Turbidity.
        :rtype: Turbidity
        """
        return Turbidity(ntu, TurbidityUnits.NTU)

    
    @property
    def ntu(self) -> float:
        """
        
        """
        if self.__ntu != None:
            return self.__ntu
        self.__ntu = self.__convert_from_base(TurbidityUnits.NTU)
        return self.__ntu

    
    def to_string(self, unit: TurbidityUnits = TurbidityUnits.NTU, fractional_digits: int = None) -> str:
        """
        Format the Turbidity to a string.
        
        Note: the default format for Turbidity is NTU.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Turbidity. Default is 'NTU'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == TurbidityUnits.NTU:
            return f"""{super()._truncate_fraction_digits(self.ntu, fractional_digits)} NTU"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: TurbidityUnits = TurbidityUnits.NTU) -> str:
        """
        Get Turbidity unit abbreviation.
        Note! the default abbreviation for Turbidity is NTU.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TurbidityUnits.NTU:
            return """NTU"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricResistanceUnits(Enum):
        """
            ElectricResistanceUnits enumeration
        """
        
        Ohm = 'Ohm'
        """
            
        """
        
        Microohm = 'Microohm'
        """
            
        """
        
        Milliohm = 'Milliohm'
        """
            
        """
        
        Kiloohm = 'Kiloohm'
        """
            
        """
        
        Megaohm = 'Megaohm'
        """
            
        """
        
        Gigaohm = 'Gigaohm'
        """
            
        """
        
        Teraohm = 'Teraohm'
        """
            
        """
        

class ElectricResistanceDto:
    """
    A DTO representation of a ElectricResistance

    Attributes:
        value (float): The value of the ElectricResistance.
        unit (ElectricResistanceUnits): The specific unit that the ElectricResistance value is representing.
    """

    def __init__(self, value: float, unit: ElectricResistanceUnits):
        """
        Create a new DTO representation of a ElectricResistance

        Parameters:
            value (float): The value of the ElectricResistance.
            unit (ElectricResistanceUnits): The specific unit that the ElectricResistance value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricResistance
        """
        self.unit: ElectricResistanceUnits = unit
        """
        The specific unit that the ElectricResistance value is representing
        """

    def to_json(self):
        """
        Get a ElectricResistance DTO JSON object representing the current unit.

        :return: JSON object represents ElectricResistance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Ohm"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricResistance DTO from a json representation.

        :param data: The ElectricResistance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Ohm"}
        :return: A new instance of ElectricResistanceDto.
        :rtype: ElectricResistanceDto
        """
        return ElectricResistanceDto(value=data["value"], unit=ElectricResistanceUnits(data["unit"]))


class ElectricResistance(AbstractMeasure):
    """
    The electrical resistance of an electrical conductor is the opposition to the passage of an electric current through that conductor.

    Args:
        value (float): The value.
        from_unit (ElectricResistanceUnits): The ElectricResistance unit to create from, The default unit is Ohm
    """
    def __init__(self, value: float, from_unit: ElectricResistanceUnits = ElectricResistanceUnits.Ohm):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__ohms = None
        
        self.__microohms = None
        
        self.__milliohms = None
        
        self.__kiloohms = None
        
        self.__megaohms = None
        
        self.__gigaohms = None
        
        self.__teraohms = None
        

    def convert(self, unit: ElectricResistanceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricResistanceUnits = ElectricResistanceUnits.Ohm) -> ElectricResistanceDto:
        """
        Get a new instance of ElectricResistance DTO representing the current unit.

        :param hold_in_unit: The specific ElectricResistance unit to store the ElectricResistance value in the DTO representation.
        :type hold_in_unit: ElectricResistanceUnits
        :return: A new instance of ElectricResistanceDto.
        :rtype: ElectricResistanceDto
        """
        return ElectricResistanceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricResistanceUnits = ElectricResistanceUnits.Ohm):
        """
        Get a ElectricResistance DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricResistance unit to store the ElectricResistance value in the DTO representation.
        :type hold_in_unit: ElectricResistanceUnits
        :return: JSON object represents ElectricResistance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Ohm"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_resistance_dto: ElectricResistanceDto):
        """
        Obtain a new instance of ElectricResistance from a DTO unit object.

        :param electric_resistance_dto: The ElectricResistance DTO representation.
        :type electric_resistance_dto: ElectricResistanceDto
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(electric_resistance_dto.value, electric_resistance_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricResistance from a DTO unit json representation.

        :param data: The ElectricResistance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Ohm"}
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance.from_dto(ElectricResistanceDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricResistanceUnits) -> float:
        value = self._value
        
        if from_unit == ElectricResistanceUnits.Ohm:
            return (value)
        
        if from_unit == ElectricResistanceUnits.Microohm:
            return ((value) / 1e-06)
        
        if from_unit == ElectricResistanceUnits.Milliohm:
            return ((value) / 0.001)
        
        if from_unit == ElectricResistanceUnits.Kiloohm:
            return ((value) / 1000.0)
        
        if from_unit == ElectricResistanceUnits.Megaohm:
            return ((value) / 1000000.0)
        
        if from_unit == ElectricResistanceUnits.Gigaohm:
            return ((value) / 1000000000.0)
        
        if from_unit == ElectricResistanceUnits.Teraohm:
            return ((value) / 1000000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricResistanceUnits) -> float:
        
        if to_unit == ElectricResistanceUnits.Ohm:
            return (value)
        
        if to_unit == ElectricResistanceUnits.Microohm:
            return ((value) * 1e-06)
        
        if to_unit == ElectricResistanceUnits.Milliohm:
            return ((value) * 0.001)
        
        if to_unit == ElectricResistanceUnits.Kiloohm:
            return ((value) * 1000.0)
        
        if to_unit == ElectricResistanceUnits.Megaohm:
            return ((value) * 1000000.0)
        
        if to_unit == ElectricResistanceUnits.Gigaohm:
            return ((value) * 1000000000.0)
        
        if to_unit == ElectricResistanceUnits.Teraohm:
            return ((value) * 1000000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_ohms(ohms: float):
        """
        Create a new instance of ElectricResistance from a value in ohms.

        

        :param meters: The ElectricResistance value in ohms.
        :type ohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(ohms, ElectricResistanceUnits.Ohm)

    
    @staticmethod
    def from_microohms(microohms: float):
        """
        Create a new instance of ElectricResistance from a value in microohms.

        

        :param meters: The ElectricResistance value in microohms.
        :type microohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(microohms, ElectricResistanceUnits.Microohm)

    
    @staticmethod
    def from_milliohms(milliohms: float):
        """
        Create a new instance of ElectricResistance from a value in milliohms.

        

        :param meters: The ElectricResistance value in milliohms.
        :type milliohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(milliohms, ElectricResistanceUnits.Milliohm)

    
    @staticmethod
    def from_kiloohms(kiloohms: float):
        """
        Create a new instance of ElectricResistance from a value in kiloohms.

        

        :param meters: The ElectricResistance value in kiloohms.
        :type kiloohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(kiloohms, ElectricResistanceUnits.Kiloohm)

    
    @staticmethod
    def from_megaohms(megaohms: float):
        """
        Create a new instance of ElectricResistance from a value in megaohms.

        

        :param meters: The ElectricResistance value in megaohms.
        :type megaohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(megaohms, ElectricResistanceUnits.Megaohm)

    
    @staticmethod
    def from_gigaohms(gigaohms: float):
        """
        Create a new instance of ElectricResistance from a value in gigaohms.

        

        :param meters: The ElectricResistance value in gigaohms.
        :type gigaohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(gigaohms, ElectricResistanceUnits.Gigaohm)

    
    @staticmethod
    def from_teraohms(teraohms: float):
        """
        Create a new instance of ElectricResistance from a value in teraohms.

        

        :param meters: The ElectricResistance value in teraohms.
        :type teraohms: float
        :return: A new instance of ElectricResistance.
        :rtype: ElectricResistance
        """
        return ElectricResistance(teraohms, ElectricResistanceUnits.Teraohm)

    
    @property
    def ohms(self) -> float:
        """
        
        """
        if self.__ohms != None:
            return self.__ohms
        self.__ohms = self.__convert_from_base(ElectricResistanceUnits.Ohm)
        return self.__ohms

    
    @property
    def microohms(self) -> float:
        """
        
        """
        if self.__microohms != None:
            return self.__microohms
        self.__microohms = self.__convert_from_base(ElectricResistanceUnits.Microohm)
        return self.__microohms

    
    @property
    def milliohms(self) -> float:
        """
        
        """
        if self.__milliohms != None:
            return self.__milliohms
        self.__milliohms = self.__convert_from_base(ElectricResistanceUnits.Milliohm)
        return self.__milliohms

    
    @property
    def kiloohms(self) -> float:
        """
        
        """
        if self.__kiloohms != None:
            return self.__kiloohms
        self.__kiloohms = self.__convert_from_base(ElectricResistanceUnits.Kiloohm)
        return self.__kiloohms

    
    @property
    def megaohms(self) -> float:
        """
        
        """
        if self.__megaohms != None:
            return self.__megaohms
        self.__megaohms = self.__convert_from_base(ElectricResistanceUnits.Megaohm)
        return self.__megaohms

    
    @property
    def gigaohms(self) -> float:
        """
        
        """
        if self.__gigaohms != None:
            return self.__gigaohms
        self.__gigaohms = self.__convert_from_base(ElectricResistanceUnits.Gigaohm)
        return self.__gigaohms

    
    @property
    def teraohms(self) -> float:
        """
        
        """
        if self.__teraohms != None:
            return self.__teraohms
        self.__teraohms = self.__convert_from_base(ElectricResistanceUnits.Teraohm)
        return self.__teraohms

    
    def to_string(self, unit: ElectricResistanceUnits = ElectricResistanceUnits.Ohm, fractional_digits: int = None) -> str:
        """
        Format the ElectricResistance to a string.
        
        Note: the default format for ElectricResistance is Ohm.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricResistance. Default is 'Ohm'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricResistanceUnits.Ohm:
            return f"""{super()._truncate_fraction_digits(self.ohms, fractional_digits)} Ω"""
        
        if unit == ElectricResistanceUnits.Microohm:
            return f"""{super()._truncate_fraction_digits(self.microohms, fractional_digits)} μΩ"""
        
        if unit == ElectricResistanceUnits.Milliohm:
            return f"""{super()._truncate_fraction_digits(self.milliohms, fractional_digits)} mΩ"""
        
        if unit == ElectricResistanceUnits.Kiloohm:
            return f"""{super()._truncate_fraction_digits(self.kiloohms, fractional_digits)} kΩ"""
        
        if unit == ElectricResistanceUnits.Megaohm:
            return f"""{super()._truncate_fraction_digits(self.megaohms, fractional_digits)} MΩ"""
        
        if unit == ElectricResistanceUnits.Gigaohm:
            return f"""{super()._truncate_fraction_digits(self.gigaohms, fractional_digits)} GΩ"""
        
        if unit == ElectricResistanceUnits.Teraohm:
            return f"""{super()._truncate_fraction_digits(self.teraohms, fractional_digits)} TΩ"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricResistanceUnits = ElectricResistanceUnits.Ohm) -> str:
        """
        Get ElectricResistance unit abbreviation.
        Note! the default abbreviation for ElectricResistance is Ohm.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricResistanceUnits.Ohm:
            return """Ω"""
        
        if unit_abbreviation == ElectricResistanceUnits.Microohm:
            return """μΩ"""
        
        if unit_abbreviation == ElectricResistanceUnits.Milliohm:
            return """mΩ"""
        
        if unit_abbreviation == ElectricResistanceUnits.Kiloohm:
            return """kΩ"""
        
        if unit_abbreviation == ElectricResistanceUnits.Megaohm:
            return """MΩ"""
        
        if unit_abbreviation == ElectricResistanceUnits.Gigaohm:
            return """GΩ"""
        
        if unit_abbreviation == ElectricResistanceUnits.Teraohm:
            return """TΩ"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricPotentialDcUnits(Enum):
        """
            ElectricPotentialDcUnits enumeration
        """
        
        VoltDc = 'VoltDc'
        """
            
        """
        
        MicrovoltDc = 'MicrovoltDc'
        """
            
        """
        
        MillivoltDc = 'MillivoltDc'
        """
            
        """
        
        KilovoltDc = 'KilovoltDc'
        """
            
        """
        
        MegavoltDc = 'MegavoltDc'
        """
            
        """
        

class ElectricPotentialDcDto:
    """
    A DTO representation of a ElectricPotentialDc

    Attributes:
        value (float): The value of the ElectricPotentialDc.
        unit (ElectricPotentialDcUnits): The specific unit that the ElectricPotentialDc value is representing.
    """

    def __init__(self, value: float, unit: ElectricPotentialDcUnits):
        """
        Create a new DTO representation of a ElectricPotentialDc

        Parameters:
            value (float): The value of the ElectricPotentialDc.
            unit (ElectricPotentialDcUnits): The specific unit that the ElectricPotentialDc value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricPotentialDc
        """
        self.unit: ElectricPotentialDcUnits = unit
        """
        The specific unit that the ElectricPotentialDc value is representing
        """

    def to_json(self):
        """
        Get a ElectricPotentialDc DTO JSON object representing the current unit.

        :return: JSON object represents ElectricPotentialDc DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltDc"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricPotentialDc DTO from a json representation.

        :param data: The ElectricPotentialDc DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltDc"}
        :return: A new instance of ElectricPotentialDcDto.
        :rtype: ElectricPotentialDcDto
        """
        return ElectricPotentialDcDto(value=data["value"], unit=ElectricPotentialDcUnits(data["unit"]))


class ElectricPotentialDc(AbstractMeasure):
    """
    The Electric Potential of a system known to use Direct Current.

    Args:
        value (float): The value.
        from_unit (ElectricPotentialDcUnits): The ElectricPotentialDc unit to create from, The default unit is VoltDc
    """
    def __init__(self, value: float, from_unit: ElectricPotentialDcUnits = ElectricPotentialDcUnits.VoltDc):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__volts_dc = None
        
        self.__microvolts_dc = None
        
        self.__millivolts_dc = None
        
        self.__kilovolts_dc = None
        
        self.__megavolts_dc = None
        

    def convert(self, unit: ElectricPotentialDcUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricPotentialDcUnits = ElectricPotentialDcUnits.VoltDc) -> ElectricPotentialDcDto:
        """
        Get a new instance of ElectricPotentialDc DTO representing the current unit.

        :param hold_in_unit: The specific ElectricPotentialDc unit to store the ElectricPotentialDc value in the DTO representation.
        :type hold_in_unit: ElectricPotentialDcUnits
        :return: A new instance of ElectricPotentialDcDto.
        :rtype: ElectricPotentialDcDto
        """
        return ElectricPotentialDcDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricPotentialDcUnits = ElectricPotentialDcUnits.VoltDc):
        """
        Get a ElectricPotentialDc DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricPotentialDc unit to store the ElectricPotentialDc value in the DTO representation.
        :type hold_in_unit: ElectricPotentialDcUnits
        :return: JSON object represents ElectricPotentialDc DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "VoltDc"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_potential_dc_dto: ElectricPotentialDcDto):
        """
        Obtain a new instance of ElectricPotentialDc from a DTO unit object.

        :param electric_potential_dc_dto: The ElectricPotentialDc DTO representation.
        :type electric_potential_dc_dto: ElectricPotentialDcDto
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(electric_potential_dc_dto.value, electric_potential_dc_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricPotentialDc from a DTO unit json representation.

        :param data: The ElectricPotentialDc DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "VoltDc"}
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc.from_dto(ElectricPotentialDcDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricPotentialDcUnits) -> float:
        value = self._value
        
        if from_unit == ElectricPotentialDcUnits.VoltDc:
            return (value)
        
        if from_unit == ElectricPotentialDcUnits.MicrovoltDc:
            return ((value) / 1e-06)
        
        if from_unit == ElectricPotentialDcUnits.MillivoltDc:
            return ((value) / 0.001)
        
        if from_unit == ElectricPotentialDcUnits.KilovoltDc:
            return ((value) / 1000.0)
        
        if from_unit == ElectricPotentialDcUnits.MegavoltDc:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricPotentialDcUnits) -> float:
        
        if to_unit == ElectricPotentialDcUnits.VoltDc:
            return (value)
        
        if to_unit == ElectricPotentialDcUnits.MicrovoltDc:
            return ((value) * 1e-06)
        
        if to_unit == ElectricPotentialDcUnits.MillivoltDc:
            return ((value) * 0.001)
        
        if to_unit == ElectricPotentialDcUnits.KilovoltDc:
            return ((value) * 1000.0)
        
        if to_unit == ElectricPotentialDcUnits.MegavoltDc:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_volts_dc(volts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in volts_dc.

        

        :param meters: The ElectricPotentialDc value in volts_dc.
        :type volts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(volts_dc, ElectricPotentialDcUnits.VoltDc)

    
    @staticmethod
    def from_microvolts_dc(microvolts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in microvolts_dc.

        

        :param meters: The ElectricPotentialDc value in microvolts_dc.
        :type microvolts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(microvolts_dc, ElectricPotentialDcUnits.MicrovoltDc)

    
    @staticmethod
    def from_millivolts_dc(millivolts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in millivolts_dc.

        

        :param meters: The ElectricPotentialDc value in millivolts_dc.
        :type millivolts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(millivolts_dc, ElectricPotentialDcUnits.MillivoltDc)

    
    @staticmethod
    def from_kilovolts_dc(kilovolts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in kilovolts_dc.

        

        :param meters: The ElectricPotentialDc value in kilovolts_dc.
        :type kilovolts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(kilovolts_dc, ElectricPotentialDcUnits.KilovoltDc)

    
    @staticmethod
    def from_megavolts_dc(megavolts_dc: float):
        """
        Create a new instance of ElectricPotentialDc from a value in megavolts_dc.

        

        :param meters: The ElectricPotentialDc value in megavolts_dc.
        :type megavolts_dc: float
        :return: A new instance of ElectricPotentialDc.
        :rtype: ElectricPotentialDc
        """
        return ElectricPotentialDc(megavolts_dc, ElectricPotentialDcUnits.MegavoltDc)

    
    @property
    def volts_dc(self) -> float:
        """
        
        """
        if self.__volts_dc != None:
            return self.__volts_dc
        self.__volts_dc = self.__convert_from_base(ElectricPotentialDcUnits.VoltDc)
        return self.__volts_dc

    
    @property
    def microvolts_dc(self) -> float:
        """
        
        """
        if self.__microvolts_dc != None:
            return self.__microvolts_dc
        self.__microvolts_dc = self.__convert_from_base(ElectricPotentialDcUnits.MicrovoltDc)
        return self.__microvolts_dc

    
    @property
    def millivolts_dc(self) -> float:
        """
        
        """
        if self.__millivolts_dc != None:
            return self.__millivolts_dc
        self.__millivolts_dc = self.__convert_from_base(ElectricPotentialDcUnits.MillivoltDc)
        return self.__millivolts_dc

    
    @property
    def kilovolts_dc(self) -> float:
        """
        
        """
        if self.__kilovolts_dc != None:
            return self.__kilovolts_dc
        self.__kilovolts_dc = self.__convert_from_base(ElectricPotentialDcUnits.KilovoltDc)
        return self.__kilovolts_dc

    
    @property
    def megavolts_dc(self) -> float:
        """
        
        """
        if self.__megavolts_dc != None:
            return self.__megavolts_dc
        self.__megavolts_dc = self.__convert_from_base(ElectricPotentialDcUnits.MegavoltDc)
        return self.__megavolts_dc

    
    def to_string(self, unit: ElectricPotentialDcUnits = ElectricPotentialDcUnits.VoltDc, fractional_digits: int = None) -> str:
        """
        Format the ElectricPotentialDc to a string.
        
        Note: the default format for ElectricPotentialDc is VoltDc.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricPotentialDc. Default is 'VoltDc'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricPotentialDcUnits.VoltDc:
            return f"""{super()._truncate_fraction_digits(self.volts_dc, fractional_digits)} Vdc"""
        
        if unit == ElectricPotentialDcUnits.MicrovoltDc:
            return f"""{super()._truncate_fraction_digits(self.microvolts_dc, fractional_digits)} μVdc"""
        
        if unit == ElectricPotentialDcUnits.MillivoltDc:
            return f"""{super()._truncate_fraction_digits(self.millivolts_dc, fractional_digits)} mVdc"""
        
        if unit == ElectricPotentialDcUnits.KilovoltDc:
            return f"""{super()._truncate_fraction_digits(self.kilovolts_dc, fractional_digits)} kVdc"""
        
        if unit == ElectricPotentialDcUnits.MegavoltDc:
            return f"""{super()._truncate_fraction_digits(self.megavolts_dc, fractional_digits)} MVdc"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricPotentialDcUnits = ElectricPotentialDcUnits.VoltDc) -> str:
        """
        Get ElectricPotentialDc unit abbreviation.
        Note! the default abbreviation for ElectricPotentialDc is VoltDc.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricPotentialDcUnits.VoltDc:
            return """Vdc"""
        
        if unit_abbreviation == ElectricPotentialDcUnits.MicrovoltDc:
            return """μVdc"""
        
        if unit_abbreviation == ElectricPotentialDcUnits.MillivoltDc:
            return """mVdc"""
        
        if unit_abbreviation == ElectricPotentialDcUnits.KilovoltDc:
            return """kVdc"""
        
        if unit_abbreviation == ElectricPotentialDcUnits.MegavoltDc:
            return """MVdc"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class StandardVolumeFlowUnits(Enum):
        """
            StandardVolumeFlowUnits enumeration
        """
        
        StandardCubicMeterPerSecond = 'StandardCubicMeterPerSecond'
        """
            
        """
        
        StandardCubicMeterPerMinute = 'StandardCubicMeterPerMinute'
        """
            
        """
        
        StandardCubicMeterPerHour = 'StandardCubicMeterPerHour'
        """
            
        """
        
        StandardCubicMeterPerDay = 'StandardCubicMeterPerDay'
        """
            
        """
        
        StandardCubicCentimeterPerMinute = 'StandardCubicCentimeterPerMinute'
        """
            
        """
        
        StandardLiterPerMinute = 'StandardLiterPerMinute'
        """
            
        """
        
        StandardCubicFootPerSecond = 'StandardCubicFootPerSecond'
        """
            
        """
        
        StandardCubicFootPerMinute = 'StandardCubicFootPerMinute'
        """
            
        """
        
        StandardCubicFootPerHour = 'StandardCubicFootPerHour'
        """
            
        """
        

class StandardVolumeFlowDto:
    """
    A DTO representation of a StandardVolumeFlow

    Attributes:
        value (float): The value of the StandardVolumeFlow.
        unit (StandardVolumeFlowUnits): The specific unit that the StandardVolumeFlow value is representing.
    """

    def __init__(self, value: float, unit: StandardVolumeFlowUnits):
        """
        Create a new DTO representation of a StandardVolumeFlow

        Parameters:
            value (float): The value of the StandardVolumeFlow.
            unit (StandardVolumeFlowUnits): The specific unit that the StandardVolumeFlow value is representing.
        """
        self.value: float = value
        """
        The value of the StandardVolumeFlow
        """
        self.unit: StandardVolumeFlowUnits = unit
        """
        The specific unit that the StandardVolumeFlow value is representing
        """

    def to_json(self):
        """
        Get a StandardVolumeFlow DTO JSON object representing the current unit.

        :return: JSON object represents StandardVolumeFlow DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "StandardCubicMeterPerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of StandardVolumeFlow DTO from a json representation.

        :param data: The StandardVolumeFlow DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "StandardCubicMeterPerSecond"}
        :return: A new instance of StandardVolumeFlowDto.
        :rtype: StandardVolumeFlowDto
        """
        return StandardVolumeFlowDto(value=data["value"], unit=StandardVolumeFlowUnits(data["unit"]))


class StandardVolumeFlow(AbstractMeasure):
    """
    The molar flow rate of a gas corrected to standardized conditions of temperature and pressure thus representing a fixed number of moles of gas regardless of composition and actual flow conditions.

    Args:
        value (float): The value.
        from_unit (StandardVolumeFlowUnits): The StandardVolumeFlow unit to create from, The default unit is StandardCubicMeterPerSecond
    """
    def __init__(self, value: float, from_unit: StandardVolumeFlowUnits = StandardVolumeFlowUnits.StandardCubicMeterPerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__standard_cubic_meters_per_second = None
        
        self.__standard_cubic_meters_per_minute = None
        
        self.__standard_cubic_meters_per_hour = None
        
        self.__standard_cubic_meters_per_day = None
        
        self.__standard_cubic_centimeters_per_minute = None
        
        self.__standard_liters_per_minute = None
        
        self.__standard_cubic_feet_per_second = None
        
        self.__standard_cubic_feet_per_minute = None
        
        self.__standard_cubic_feet_per_hour = None
        

    def convert(self, unit: StandardVolumeFlowUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: StandardVolumeFlowUnits = StandardVolumeFlowUnits.StandardCubicMeterPerSecond) -> StandardVolumeFlowDto:
        """
        Get a new instance of StandardVolumeFlow DTO representing the current unit.

        :param hold_in_unit: The specific StandardVolumeFlow unit to store the StandardVolumeFlow value in the DTO representation.
        :type hold_in_unit: StandardVolumeFlowUnits
        :return: A new instance of StandardVolumeFlowDto.
        :rtype: StandardVolumeFlowDto
        """
        return StandardVolumeFlowDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: StandardVolumeFlowUnits = StandardVolumeFlowUnits.StandardCubicMeterPerSecond):
        """
        Get a StandardVolumeFlow DTO JSON object representing the current unit.

        :param hold_in_unit: The specific StandardVolumeFlow unit to store the StandardVolumeFlow value in the DTO representation.
        :type hold_in_unit: StandardVolumeFlowUnits
        :return: JSON object represents StandardVolumeFlow DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "StandardCubicMeterPerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(standard_volume_flow_dto: StandardVolumeFlowDto):
        """
        Obtain a new instance of StandardVolumeFlow from a DTO unit object.

        :param standard_volume_flow_dto: The StandardVolumeFlow DTO representation.
        :type standard_volume_flow_dto: StandardVolumeFlowDto
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_volume_flow_dto.value, standard_volume_flow_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of StandardVolumeFlow from a DTO unit json representation.

        :param data: The StandardVolumeFlow DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "StandardCubicMeterPerSecond"}
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow.from_dto(StandardVolumeFlowDto.from_json(data))

    def __convert_from_base(self, from_unit: StandardVolumeFlowUnits) -> float:
        value = self._value
        
        if from_unit == StandardVolumeFlowUnits.StandardCubicMeterPerSecond:
            return (value)
        
        if from_unit == StandardVolumeFlowUnits.StandardCubicMeterPerMinute:
            return (value * 60)
        
        if from_unit == StandardVolumeFlowUnits.StandardCubicMeterPerHour:
            return (value * 3600)
        
        if from_unit == StandardVolumeFlowUnits.StandardCubicMeterPerDay:
            return (value * 86400)
        
        if from_unit == StandardVolumeFlowUnits.StandardCubicCentimeterPerMinute:
            return (value * 6e7)
        
        if from_unit == StandardVolumeFlowUnits.StandardLiterPerMinute:
            return (value * 60000)
        
        if from_unit == StandardVolumeFlowUnits.StandardCubicFootPerSecond:
            return (value * 35.314666721)
        
        if from_unit == StandardVolumeFlowUnits.StandardCubicFootPerMinute:
            return (value * 2118.88000326)
        
        if from_unit == StandardVolumeFlowUnits.StandardCubicFootPerHour:
            return (value / 7.8657907199999087346816086183876e-6)
        
        return None


    def __convert_to_base(self, value: float, to_unit: StandardVolumeFlowUnits) -> float:
        
        if to_unit == StandardVolumeFlowUnits.StandardCubicMeterPerSecond:
            return (value)
        
        if to_unit == StandardVolumeFlowUnits.StandardCubicMeterPerMinute:
            return (value / 60)
        
        if to_unit == StandardVolumeFlowUnits.StandardCubicMeterPerHour:
            return (value / 3600)
        
        if to_unit == StandardVolumeFlowUnits.StandardCubicMeterPerDay:
            return (value / 86400)
        
        if to_unit == StandardVolumeFlowUnits.StandardCubicCentimeterPerMinute:
            return (value / 6e7)
        
        if to_unit == StandardVolumeFlowUnits.StandardLiterPerMinute:
            return (value / 60000)
        
        if to_unit == StandardVolumeFlowUnits.StandardCubicFootPerSecond:
            return (value / 35.314666721)
        
        if to_unit == StandardVolumeFlowUnits.StandardCubicFootPerMinute:
            return (value / 2118.88000326)
        
        if to_unit == StandardVolumeFlowUnits.StandardCubicFootPerHour:
            return (value * 7.8657907199999087346816086183876e-6)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_standard_cubic_meters_per_second(standard_cubic_meters_per_second: float):
        """
        Create a new instance of StandardVolumeFlow from a value in standard_cubic_meters_per_second.

        

        :param meters: The StandardVolumeFlow value in standard_cubic_meters_per_second.
        :type standard_cubic_meters_per_second: float
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_cubic_meters_per_second, StandardVolumeFlowUnits.StandardCubicMeterPerSecond)

    
    @staticmethod
    def from_standard_cubic_meters_per_minute(standard_cubic_meters_per_minute: float):
        """
        Create a new instance of StandardVolumeFlow from a value in standard_cubic_meters_per_minute.

        

        :param meters: The StandardVolumeFlow value in standard_cubic_meters_per_minute.
        :type standard_cubic_meters_per_minute: float
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_cubic_meters_per_minute, StandardVolumeFlowUnits.StandardCubicMeterPerMinute)

    
    @staticmethod
    def from_standard_cubic_meters_per_hour(standard_cubic_meters_per_hour: float):
        """
        Create a new instance of StandardVolumeFlow from a value in standard_cubic_meters_per_hour.

        

        :param meters: The StandardVolumeFlow value in standard_cubic_meters_per_hour.
        :type standard_cubic_meters_per_hour: float
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_cubic_meters_per_hour, StandardVolumeFlowUnits.StandardCubicMeterPerHour)

    
    @staticmethod
    def from_standard_cubic_meters_per_day(standard_cubic_meters_per_day: float):
        """
        Create a new instance of StandardVolumeFlow from a value in standard_cubic_meters_per_day.

        

        :param meters: The StandardVolumeFlow value in standard_cubic_meters_per_day.
        :type standard_cubic_meters_per_day: float
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_cubic_meters_per_day, StandardVolumeFlowUnits.StandardCubicMeterPerDay)

    
    @staticmethod
    def from_standard_cubic_centimeters_per_minute(standard_cubic_centimeters_per_minute: float):
        """
        Create a new instance of StandardVolumeFlow from a value in standard_cubic_centimeters_per_minute.

        

        :param meters: The StandardVolumeFlow value in standard_cubic_centimeters_per_minute.
        :type standard_cubic_centimeters_per_minute: float
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_cubic_centimeters_per_minute, StandardVolumeFlowUnits.StandardCubicCentimeterPerMinute)

    
    @staticmethod
    def from_standard_liters_per_minute(standard_liters_per_minute: float):
        """
        Create a new instance of StandardVolumeFlow from a value in standard_liters_per_minute.

        

        :param meters: The StandardVolumeFlow value in standard_liters_per_minute.
        :type standard_liters_per_minute: float
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_liters_per_minute, StandardVolumeFlowUnits.StandardLiterPerMinute)

    
    @staticmethod
    def from_standard_cubic_feet_per_second(standard_cubic_feet_per_second: float):
        """
        Create a new instance of StandardVolumeFlow from a value in standard_cubic_feet_per_second.

        

        :param meters: The StandardVolumeFlow value in standard_cubic_feet_per_second.
        :type standard_cubic_feet_per_second: float
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_cubic_feet_per_second, StandardVolumeFlowUnits.StandardCubicFootPerSecond)

    
    @staticmethod
    def from_standard_cubic_feet_per_minute(standard_cubic_feet_per_minute: float):
        """
        Create a new instance of StandardVolumeFlow from a value in standard_cubic_feet_per_minute.

        

        :param meters: The StandardVolumeFlow value in standard_cubic_feet_per_minute.
        :type standard_cubic_feet_per_minute: float
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_cubic_feet_per_minute, StandardVolumeFlowUnits.StandardCubicFootPerMinute)

    
    @staticmethod
    def from_standard_cubic_feet_per_hour(standard_cubic_feet_per_hour: float):
        """
        Create a new instance of StandardVolumeFlow from a value in standard_cubic_feet_per_hour.

        

        :param meters: The StandardVolumeFlow value in standard_cubic_feet_per_hour.
        :type standard_cubic_feet_per_hour: float
        :return: A new instance of StandardVolumeFlow.
        :rtype: StandardVolumeFlow
        """
        return StandardVolumeFlow(standard_cubic_feet_per_hour, StandardVolumeFlowUnits.StandardCubicFootPerHour)

    
    @property
    def standard_cubic_meters_per_second(self) -> float:
        """
        
        """
        if self.__standard_cubic_meters_per_second != None:
            return self.__standard_cubic_meters_per_second
        self.__standard_cubic_meters_per_second = self.__convert_from_base(StandardVolumeFlowUnits.StandardCubicMeterPerSecond)
        return self.__standard_cubic_meters_per_second

    
    @property
    def standard_cubic_meters_per_minute(self) -> float:
        """
        
        """
        if self.__standard_cubic_meters_per_minute != None:
            return self.__standard_cubic_meters_per_minute
        self.__standard_cubic_meters_per_minute = self.__convert_from_base(StandardVolumeFlowUnits.StandardCubicMeterPerMinute)
        return self.__standard_cubic_meters_per_minute

    
    @property
    def standard_cubic_meters_per_hour(self) -> float:
        """
        
        """
        if self.__standard_cubic_meters_per_hour != None:
            return self.__standard_cubic_meters_per_hour
        self.__standard_cubic_meters_per_hour = self.__convert_from_base(StandardVolumeFlowUnits.StandardCubicMeterPerHour)
        return self.__standard_cubic_meters_per_hour

    
    @property
    def standard_cubic_meters_per_day(self) -> float:
        """
        
        """
        if self.__standard_cubic_meters_per_day != None:
            return self.__standard_cubic_meters_per_day
        self.__standard_cubic_meters_per_day = self.__convert_from_base(StandardVolumeFlowUnits.StandardCubicMeterPerDay)
        return self.__standard_cubic_meters_per_day

    
    @property
    def standard_cubic_centimeters_per_minute(self) -> float:
        """
        
        """
        if self.__standard_cubic_centimeters_per_minute != None:
            return self.__standard_cubic_centimeters_per_minute
        self.__standard_cubic_centimeters_per_minute = self.__convert_from_base(StandardVolumeFlowUnits.StandardCubicCentimeterPerMinute)
        return self.__standard_cubic_centimeters_per_minute

    
    @property
    def standard_liters_per_minute(self) -> float:
        """
        
        """
        if self.__standard_liters_per_minute != None:
            return self.__standard_liters_per_minute
        self.__standard_liters_per_minute = self.__convert_from_base(StandardVolumeFlowUnits.StandardLiterPerMinute)
        return self.__standard_liters_per_minute

    
    @property
    def standard_cubic_feet_per_second(self) -> float:
        """
        
        """
        if self.__standard_cubic_feet_per_second != None:
            return self.__standard_cubic_feet_per_second
        self.__standard_cubic_feet_per_second = self.__convert_from_base(StandardVolumeFlowUnits.StandardCubicFootPerSecond)
        return self.__standard_cubic_feet_per_second

    
    @property
    def standard_cubic_feet_per_minute(self) -> float:
        """
        
        """
        if self.__standard_cubic_feet_per_minute != None:
            return self.__standard_cubic_feet_per_minute
        self.__standard_cubic_feet_per_minute = self.__convert_from_base(StandardVolumeFlowUnits.StandardCubicFootPerMinute)
        return self.__standard_cubic_feet_per_minute

    
    @property
    def standard_cubic_feet_per_hour(self) -> float:
        """
        
        """
        if self.__standard_cubic_feet_per_hour != None:
            return self.__standard_cubic_feet_per_hour
        self.__standard_cubic_feet_per_hour = self.__convert_from_base(StandardVolumeFlowUnits.StandardCubicFootPerHour)
        return self.__standard_cubic_feet_per_hour

    
    def to_string(self, unit: StandardVolumeFlowUnits = StandardVolumeFlowUnits.StandardCubicMeterPerSecond, fractional_digits: int = None) -> str:
        """
        Format the StandardVolumeFlow to a string.
        
        Note: the default format for StandardVolumeFlow is StandardCubicMeterPerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the StandardVolumeFlow. Default is 'StandardCubicMeterPerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == StandardVolumeFlowUnits.StandardCubicMeterPerSecond:
            return f"""{super()._truncate_fraction_digits(self.standard_cubic_meters_per_second, fractional_digits)} Sm³/s"""
        
        if unit == StandardVolumeFlowUnits.StandardCubicMeterPerMinute:
            return f"""{super()._truncate_fraction_digits(self.standard_cubic_meters_per_minute, fractional_digits)} Sm³/min"""
        
        if unit == StandardVolumeFlowUnits.StandardCubicMeterPerHour:
            return f"""{super()._truncate_fraction_digits(self.standard_cubic_meters_per_hour, fractional_digits)} Sm³/h"""
        
        if unit == StandardVolumeFlowUnits.StandardCubicMeterPerDay:
            return f"""{super()._truncate_fraction_digits(self.standard_cubic_meters_per_day, fractional_digits)} Sm³/d"""
        
        if unit == StandardVolumeFlowUnits.StandardCubicCentimeterPerMinute:
            return f"""{super()._truncate_fraction_digits(self.standard_cubic_centimeters_per_minute, fractional_digits)} sccm"""
        
        if unit == StandardVolumeFlowUnits.StandardLiterPerMinute:
            return f"""{super()._truncate_fraction_digits(self.standard_liters_per_minute, fractional_digits)} slm"""
        
        if unit == StandardVolumeFlowUnits.StandardCubicFootPerSecond:
            return f"""{super()._truncate_fraction_digits(self.standard_cubic_feet_per_second, fractional_digits)} Sft³/s"""
        
        if unit == StandardVolumeFlowUnits.StandardCubicFootPerMinute:
            return f"""{super()._truncate_fraction_digits(self.standard_cubic_feet_per_minute, fractional_digits)} scfm"""
        
        if unit == StandardVolumeFlowUnits.StandardCubicFootPerHour:
            return f"""{super()._truncate_fraction_digits(self.standard_cubic_feet_per_hour, fractional_digits)} scfh"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: StandardVolumeFlowUnits = StandardVolumeFlowUnits.StandardCubicMeterPerSecond) -> str:
        """
        Get StandardVolumeFlow unit abbreviation.
        Note! the default abbreviation for StandardVolumeFlow is StandardCubicMeterPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == StandardVolumeFlowUnits.StandardCubicMeterPerSecond:
            return """Sm³/s"""
        
        if unit_abbreviation == StandardVolumeFlowUnits.StandardCubicMeterPerMinute:
            return """Sm³/min"""
        
        if unit_abbreviation == StandardVolumeFlowUnits.StandardCubicMeterPerHour:
            return """Sm³/h"""
        
        if unit_abbreviation == StandardVolumeFlowUnits.StandardCubicMeterPerDay:
            return """Sm³/d"""
        
        if unit_abbreviation == StandardVolumeFlowUnits.StandardCubicCentimeterPerMinute:
            return """sccm"""
        
        if unit_abbreviation == StandardVolumeFlowUnits.StandardLiterPerMinute:
            return """slm"""
        
        if unit_abbreviation == StandardVolumeFlowUnits.StandardCubicFootPerSecond:
            return """Sft³/s"""
        
        if unit_abbreviation == StandardVolumeFlowUnits.StandardCubicFootPerMinute:
            return """scfm"""
        
        if unit_abbreviation == StandardVolumeFlowUnits.StandardCubicFootPerHour:
            return """scfh"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class IlluminanceUnits(Enum):
        """
            IlluminanceUnits enumeration
        """
        
        Lux = 'Lux'
        """
            
        """
        
        Millilux = 'Millilux'
        """
            
        """
        
        Kilolux = 'Kilolux'
        """
            
        """
        
        Megalux = 'Megalux'
        """
            
        """
        

class IlluminanceDto:
    """
    A DTO representation of a Illuminance

    Attributes:
        value (float): The value of the Illuminance.
        unit (IlluminanceUnits): The specific unit that the Illuminance value is representing.
    """

    def __init__(self, value: float, unit: IlluminanceUnits):
        """
        Create a new DTO representation of a Illuminance

        Parameters:
            value (float): The value of the Illuminance.
            unit (IlluminanceUnits): The specific unit that the Illuminance value is representing.
        """
        self.value: float = value
        """
        The value of the Illuminance
        """
        self.unit: IlluminanceUnits = unit
        """
        The specific unit that the Illuminance value is representing
        """

    def to_json(self):
        """
        Get a Illuminance DTO JSON object representing the current unit.

        :return: JSON object represents Illuminance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Lux"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Illuminance DTO from a json representation.

        :param data: The Illuminance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Lux"}
        :return: A new instance of IlluminanceDto.
        :rtype: IlluminanceDto
        """
        return IlluminanceDto(value=data["value"], unit=IlluminanceUnits(data["unit"]))


class Illuminance(AbstractMeasure):
    """
    In photometry, illuminance is the total luminous flux incident on a surface, per unit area.

    Args:
        value (float): The value.
        from_unit (IlluminanceUnits): The Illuminance unit to create from, The default unit is Lux
    """
    def __init__(self, value: float, from_unit: IlluminanceUnits = IlluminanceUnits.Lux):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__lux = None
        
        self.__millilux = None
        
        self.__kilolux = None
        
        self.__megalux = None
        

    def convert(self, unit: IlluminanceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: IlluminanceUnits = IlluminanceUnits.Lux) -> IlluminanceDto:
        """
        Get a new instance of Illuminance DTO representing the current unit.

        :param hold_in_unit: The specific Illuminance unit to store the Illuminance value in the DTO representation.
        :type hold_in_unit: IlluminanceUnits
        :return: A new instance of IlluminanceDto.
        :rtype: IlluminanceDto
        """
        return IlluminanceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: IlluminanceUnits = IlluminanceUnits.Lux):
        """
        Get a Illuminance DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Illuminance unit to store the Illuminance value in the DTO representation.
        :type hold_in_unit: IlluminanceUnits
        :return: JSON object represents Illuminance DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Lux"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(illuminance_dto: IlluminanceDto):
        """
        Obtain a new instance of Illuminance from a DTO unit object.

        :param illuminance_dto: The Illuminance DTO representation.
        :type illuminance_dto: IlluminanceDto
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance(illuminance_dto.value, illuminance_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Illuminance from a DTO unit json representation.

        :param data: The Illuminance DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Lux"}
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance.from_dto(IlluminanceDto.from_json(data))

    def __convert_from_base(self, from_unit: IlluminanceUnits) -> float:
        value = self._value
        
        if from_unit == IlluminanceUnits.Lux:
            return (value)
        
        if from_unit == IlluminanceUnits.Millilux:
            return ((value) / 0.001)
        
        if from_unit == IlluminanceUnits.Kilolux:
            return ((value) / 1000.0)
        
        if from_unit == IlluminanceUnits.Megalux:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: IlluminanceUnits) -> float:
        
        if to_unit == IlluminanceUnits.Lux:
            return (value)
        
        if to_unit == IlluminanceUnits.Millilux:
            return ((value) * 0.001)
        
        if to_unit == IlluminanceUnits.Kilolux:
            return ((value) * 1000.0)
        
        if to_unit == IlluminanceUnits.Megalux:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_lux(lux: float):
        """
        Create a new instance of Illuminance from a value in lux.

        

        :param meters: The Illuminance value in lux.
        :type lux: float
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance(lux, IlluminanceUnits.Lux)

    
    @staticmethod
    def from_millilux(millilux: float):
        """
        Create a new instance of Illuminance from a value in millilux.

        

        :param meters: The Illuminance value in millilux.
        :type millilux: float
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance(millilux, IlluminanceUnits.Millilux)

    
    @staticmethod
    def from_kilolux(kilolux: float):
        """
        Create a new instance of Illuminance from a value in kilolux.

        

        :param meters: The Illuminance value in kilolux.
        :type kilolux: float
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance(kilolux, IlluminanceUnits.Kilolux)

    
    @staticmethod
    def from_megalux(megalux: float):
        """
        Create a new instance of Illuminance from a value in megalux.

        

        :param meters: The Illuminance value in megalux.
        :type megalux: float
        :return: A new instance of Illuminance.
        :rtype: Illuminance
        """
        return Illuminance(megalux, IlluminanceUnits.Megalux)

    
    @property
    def lux(self) -> float:
        """
        
        """
        if self.__lux != None:
            return self.__lux
        self.__lux = self.__convert_from_base(IlluminanceUnits.Lux)
        return self.__lux

    
    @property
    def millilux(self) -> float:
        """
        
        """
        if self.__millilux != None:
            return self.__millilux
        self.__millilux = self.__convert_from_base(IlluminanceUnits.Millilux)
        return self.__millilux

    
    @property
    def kilolux(self) -> float:
        """
        
        """
        if self.__kilolux != None:
            return self.__kilolux
        self.__kilolux = self.__convert_from_base(IlluminanceUnits.Kilolux)
        return self.__kilolux

    
    @property
    def megalux(self) -> float:
        """
        
        """
        if self.__megalux != None:
            return self.__megalux
        self.__megalux = self.__convert_from_base(IlluminanceUnits.Megalux)
        return self.__megalux

    
    def to_string(self, unit: IlluminanceUnits = IlluminanceUnits.Lux, fractional_digits: int = None) -> str:
        """
        Format the Illuminance to a string.
        
        Note: the default format for Illuminance is Lux.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Illuminance. Default is 'Lux'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == IlluminanceUnits.Lux:
            return f"""{super()._truncate_fraction_digits(self.lux, fractional_digits)} lx"""
        
        if unit == IlluminanceUnits.Millilux:
            return f"""{super()._truncate_fraction_digits(self.millilux, fractional_digits)} mlx"""
        
        if unit == IlluminanceUnits.Kilolux:
            return f"""{super()._truncate_fraction_digits(self.kilolux, fractional_digits)} klx"""
        
        if unit == IlluminanceUnits.Megalux:
            return f"""{super()._truncate_fraction_digits(self.megalux, fractional_digits)} Mlx"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: IlluminanceUnits = IlluminanceUnits.Lux) -> str:
        """
        Get Illuminance unit abbreviation.
        Note! the default abbreviation for Illuminance is Lux.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == IlluminanceUnits.Lux:
            return """lx"""
        
        if unit_abbreviation == IlluminanceUnits.Millilux:
            return """mlx"""
        
        if unit_abbreviation == IlluminanceUnits.Kilolux:
            return """klx"""
        
        if unit_abbreviation == IlluminanceUnits.Megalux:
            return """Mlx"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricCurrentGradientUnits(Enum):
        """
            ElectricCurrentGradientUnits enumeration
        """
        
        AmperePerSecond = 'AmperePerSecond'
        """
            
        """
        
        AmperePerMinute = 'AmperePerMinute'
        """
            
        """
        
        AmperePerMillisecond = 'AmperePerMillisecond'
        """
            
        """
        
        AmperePerMicrosecond = 'AmperePerMicrosecond'
        """
            
        """
        
        AmperePerNanosecond = 'AmperePerNanosecond'
        """
            
        """
        
        MilliamperePerSecond = 'MilliamperePerSecond'
        """
            
        """
        
        MilliamperePerMinute = 'MilliamperePerMinute'
        """
            
        """
        

class ElectricCurrentGradientDto:
    """
    A DTO representation of a ElectricCurrentGradient

    Attributes:
        value (float): The value of the ElectricCurrentGradient.
        unit (ElectricCurrentGradientUnits): The specific unit that the ElectricCurrentGradient value is representing.
    """

    def __init__(self, value: float, unit: ElectricCurrentGradientUnits):
        """
        Create a new DTO representation of a ElectricCurrentGradient

        Parameters:
            value (float): The value of the ElectricCurrentGradient.
            unit (ElectricCurrentGradientUnits): The specific unit that the ElectricCurrentGradient value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricCurrentGradient
        """
        self.unit: ElectricCurrentGradientUnits = unit
        """
        The specific unit that the ElectricCurrentGradient value is representing
        """

    def to_json(self):
        """
        Get a ElectricCurrentGradient DTO JSON object representing the current unit.

        :return: JSON object represents ElectricCurrentGradient DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "AmperePerSecond"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricCurrentGradient DTO from a json representation.

        :param data: The ElectricCurrentGradient DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "AmperePerSecond"}
        :return: A new instance of ElectricCurrentGradientDto.
        :rtype: ElectricCurrentGradientDto
        """
        return ElectricCurrentGradientDto(value=data["value"], unit=ElectricCurrentGradientUnits(data["unit"]))


class ElectricCurrentGradient(AbstractMeasure):
    """
    In electromagnetism, the current gradient describes how the current changes in time.

    Args:
        value (float): The value.
        from_unit (ElectricCurrentGradientUnits): The ElectricCurrentGradient unit to create from, The default unit is AmperePerSecond
    """
    def __init__(self, value: float, from_unit: ElectricCurrentGradientUnits = ElectricCurrentGradientUnits.AmperePerSecond):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__amperes_per_second = None
        
        self.__amperes_per_minute = None
        
        self.__amperes_per_millisecond = None
        
        self.__amperes_per_microsecond = None
        
        self.__amperes_per_nanosecond = None
        
        self.__milliamperes_per_second = None
        
        self.__milliamperes_per_minute = None
        

    def convert(self, unit: ElectricCurrentGradientUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricCurrentGradientUnits = ElectricCurrentGradientUnits.AmperePerSecond) -> ElectricCurrentGradientDto:
        """
        Get a new instance of ElectricCurrentGradient DTO representing the current unit.

        :param hold_in_unit: The specific ElectricCurrentGradient unit to store the ElectricCurrentGradient value in the DTO representation.
        :type hold_in_unit: ElectricCurrentGradientUnits
        :return: A new instance of ElectricCurrentGradientDto.
        :rtype: ElectricCurrentGradientDto
        """
        return ElectricCurrentGradientDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricCurrentGradientUnits = ElectricCurrentGradientUnits.AmperePerSecond):
        """
        Get a ElectricCurrentGradient DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricCurrentGradient unit to store the ElectricCurrentGradient value in the DTO representation.
        :type hold_in_unit: ElectricCurrentGradientUnits
        :return: JSON object represents ElectricCurrentGradient DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "AmperePerSecond"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_current_gradient_dto: ElectricCurrentGradientDto):
        """
        Obtain a new instance of ElectricCurrentGradient from a DTO unit object.

        :param electric_current_gradient_dto: The ElectricCurrentGradient DTO representation.
        :type electric_current_gradient_dto: ElectricCurrentGradientDto
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(electric_current_gradient_dto.value, electric_current_gradient_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricCurrentGradient from a DTO unit json representation.

        :param data: The ElectricCurrentGradient DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "AmperePerSecond"}
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient.from_dto(ElectricCurrentGradientDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricCurrentGradientUnits) -> float:
        value = self._value
        
        if from_unit == ElectricCurrentGradientUnits.AmperePerSecond:
            return (value)
        
        if from_unit == ElectricCurrentGradientUnits.AmperePerMinute:
            return (value * 60)
        
        if from_unit == ElectricCurrentGradientUnits.AmperePerMillisecond:
            return (value / 1e3)
        
        if from_unit == ElectricCurrentGradientUnits.AmperePerMicrosecond:
            return (value / 1e6)
        
        if from_unit == ElectricCurrentGradientUnits.AmperePerNanosecond:
            return (value / 1e9)
        
        if from_unit == ElectricCurrentGradientUnits.MilliamperePerSecond:
            return ((value) / 0.001)
        
        if from_unit == ElectricCurrentGradientUnits.MilliamperePerMinute:
            return ((value * 60) / 0.001)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricCurrentGradientUnits) -> float:
        
        if to_unit == ElectricCurrentGradientUnits.AmperePerSecond:
            return (value)
        
        if to_unit == ElectricCurrentGradientUnits.AmperePerMinute:
            return (value / 60)
        
        if to_unit == ElectricCurrentGradientUnits.AmperePerMillisecond:
            return (value * 1e3)
        
        if to_unit == ElectricCurrentGradientUnits.AmperePerMicrosecond:
            return (value * 1e6)
        
        if to_unit == ElectricCurrentGradientUnits.AmperePerNanosecond:
            return (value * 1e9)
        
        if to_unit == ElectricCurrentGradientUnits.MilliamperePerSecond:
            return ((value) * 0.001)
        
        if to_unit == ElectricCurrentGradientUnits.MilliamperePerMinute:
            return ((value / 60) * 0.001)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_amperes_per_second(amperes_per_second: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in amperes_per_second.

        

        :param meters: The ElectricCurrentGradient value in amperes_per_second.
        :type amperes_per_second: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(amperes_per_second, ElectricCurrentGradientUnits.AmperePerSecond)

    
    @staticmethod
    def from_amperes_per_minute(amperes_per_minute: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in amperes_per_minute.

        

        :param meters: The ElectricCurrentGradient value in amperes_per_minute.
        :type amperes_per_minute: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(amperes_per_minute, ElectricCurrentGradientUnits.AmperePerMinute)

    
    @staticmethod
    def from_amperes_per_millisecond(amperes_per_millisecond: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in amperes_per_millisecond.

        

        :param meters: The ElectricCurrentGradient value in amperes_per_millisecond.
        :type amperes_per_millisecond: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(amperes_per_millisecond, ElectricCurrentGradientUnits.AmperePerMillisecond)

    
    @staticmethod
    def from_amperes_per_microsecond(amperes_per_microsecond: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in amperes_per_microsecond.

        

        :param meters: The ElectricCurrentGradient value in amperes_per_microsecond.
        :type amperes_per_microsecond: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(amperes_per_microsecond, ElectricCurrentGradientUnits.AmperePerMicrosecond)

    
    @staticmethod
    def from_amperes_per_nanosecond(amperes_per_nanosecond: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in amperes_per_nanosecond.

        

        :param meters: The ElectricCurrentGradient value in amperes_per_nanosecond.
        :type amperes_per_nanosecond: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(amperes_per_nanosecond, ElectricCurrentGradientUnits.AmperePerNanosecond)

    
    @staticmethod
    def from_milliamperes_per_second(milliamperes_per_second: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in milliamperes_per_second.

        

        :param meters: The ElectricCurrentGradient value in milliamperes_per_second.
        :type milliamperes_per_second: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(milliamperes_per_second, ElectricCurrentGradientUnits.MilliamperePerSecond)

    
    @staticmethod
    def from_milliamperes_per_minute(milliamperes_per_minute: float):
        """
        Create a new instance of ElectricCurrentGradient from a value in milliamperes_per_minute.

        

        :param meters: The ElectricCurrentGradient value in milliamperes_per_minute.
        :type milliamperes_per_minute: float
        :return: A new instance of ElectricCurrentGradient.
        :rtype: ElectricCurrentGradient
        """
        return ElectricCurrentGradient(milliamperes_per_minute, ElectricCurrentGradientUnits.MilliamperePerMinute)

    
    @property
    def amperes_per_second(self) -> float:
        """
        
        """
        if self.__amperes_per_second != None:
            return self.__amperes_per_second
        self.__amperes_per_second = self.__convert_from_base(ElectricCurrentGradientUnits.AmperePerSecond)
        return self.__amperes_per_second

    
    @property
    def amperes_per_minute(self) -> float:
        """
        
        """
        if self.__amperes_per_minute != None:
            return self.__amperes_per_minute
        self.__amperes_per_minute = self.__convert_from_base(ElectricCurrentGradientUnits.AmperePerMinute)
        return self.__amperes_per_minute

    
    @property
    def amperes_per_millisecond(self) -> float:
        """
        
        """
        if self.__amperes_per_millisecond != None:
            return self.__amperes_per_millisecond
        self.__amperes_per_millisecond = self.__convert_from_base(ElectricCurrentGradientUnits.AmperePerMillisecond)
        return self.__amperes_per_millisecond

    
    @property
    def amperes_per_microsecond(self) -> float:
        """
        
        """
        if self.__amperes_per_microsecond != None:
            return self.__amperes_per_microsecond
        self.__amperes_per_microsecond = self.__convert_from_base(ElectricCurrentGradientUnits.AmperePerMicrosecond)
        return self.__amperes_per_microsecond

    
    @property
    def amperes_per_nanosecond(self) -> float:
        """
        
        """
        if self.__amperes_per_nanosecond != None:
            return self.__amperes_per_nanosecond
        self.__amperes_per_nanosecond = self.__convert_from_base(ElectricCurrentGradientUnits.AmperePerNanosecond)
        return self.__amperes_per_nanosecond

    
    @property
    def milliamperes_per_second(self) -> float:
        """
        
        """
        if self.__milliamperes_per_second != None:
            return self.__milliamperes_per_second
        self.__milliamperes_per_second = self.__convert_from_base(ElectricCurrentGradientUnits.MilliamperePerSecond)
        return self.__milliamperes_per_second

    
    @property
    def milliamperes_per_minute(self) -> float:
        """
        
        """
        if self.__milliamperes_per_minute != None:
            return self.__milliamperes_per_minute
        self.__milliamperes_per_minute = self.__convert_from_base(ElectricCurrentGradientUnits.MilliamperePerMinute)
        return self.__milliamperes_per_minute

    
    def to_string(self, unit: ElectricCurrentGradientUnits = ElectricCurrentGradientUnits.AmperePerSecond, fractional_digits: int = None) -> str:
        """
        Format the ElectricCurrentGradient to a string.
        
        Note: the default format for ElectricCurrentGradient is AmperePerSecond.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricCurrentGradient. Default is 'AmperePerSecond'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricCurrentGradientUnits.AmperePerSecond:
            return f"""{super()._truncate_fraction_digits(self.amperes_per_second, fractional_digits)} A/s"""
        
        if unit == ElectricCurrentGradientUnits.AmperePerMinute:
            return f"""{super()._truncate_fraction_digits(self.amperes_per_minute, fractional_digits)} A/min"""
        
        if unit == ElectricCurrentGradientUnits.AmperePerMillisecond:
            return f"""{super()._truncate_fraction_digits(self.amperes_per_millisecond, fractional_digits)} A/ms"""
        
        if unit == ElectricCurrentGradientUnits.AmperePerMicrosecond:
            return f"""{super()._truncate_fraction_digits(self.amperes_per_microsecond, fractional_digits)} A/μs"""
        
        if unit == ElectricCurrentGradientUnits.AmperePerNanosecond:
            return f"""{super()._truncate_fraction_digits(self.amperes_per_nanosecond, fractional_digits)} A/ns"""
        
        if unit == ElectricCurrentGradientUnits.MilliamperePerSecond:
            return f"""{super()._truncate_fraction_digits(self.milliamperes_per_second, fractional_digits)} mA/s"""
        
        if unit == ElectricCurrentGradientUnits.MilliamperePerMinute:
            return f"""{super()._truncate_fraction_digits(self.milliamperes_per_minute, fractional_digits)} mA/min"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricCurrentGradientUnits = ElectricCurrentGradientUnits.AmperePerSecond) -> str:
        """
        Get ElectricCurrentGradient unit abbreviation.
        Note! the default abbreviation for ElectricCurrentGradient is AmperePerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricCurrentGradientUnits.AmperePerSecond:
            return """A/s"""
        
        if unit_abbreviation == ElectricCurrentGradientUnits.AmperePerMinute:
            return """A/min"""
        
        if unit_abbreviation == ElectricCurrentGradientUnits.AmperePerMillisecond:
            return """A/ms"""
        
        if unit_abbreviation == ElectricCurrentGradientUnits.AmperePerMicrosecond:
            return """A/μs"""
        
        if unit_abbreviation == ElectricCurrentGradientUnits.AmperePerNanosecond:
            return """A/ns"""
        
        if unit_abbreviation == ElectricCurrentGradientUnits.MilliamperePerSecond:
            return """mA/s"""
        
        if unit_abbreviation == ElectricCurrentGradientUnits.MilliamperePerMinute:
            return """mA/min"""
        